

#include "error.h"


#include "kvbuffer.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#ifdef WIN32
#define snprintf _snprintf
#endif


KVBuffer::KVBuffer()
:nbPairs_(0)
{
}
KVBuffer::~KVBuffer()
{
}

void KVBuffer::reset()
{
  nbPairs_ = 0;
}

bool KVBuffer::initialize(const char* ibuf)
{
  nbPairs_ = 0;
  
  debug("KVBuffer::initialize");
  
  int bufLen = strlen(ibuf);
  
  const char* first = ibuf;
  const char* last = ibuf;
  bool end = false;
  while(first[0] != 0 && !end)
  {
    if (nbPairs_ >= MAX_NB_PAIRS-1)
    {
      debug("KVBuffer: MAX_NB_PAIRS=%d overloaded! Aborting", MAX_NB_PAIRS);
      break;
    }
    
    last = strchr(last, '=');
    if (last == (char*)-1)
    {
      debug("last == -1 !!! End in the middle of the buffer");
      end = true;
      break;
    }
    memcpy(pairs_[nbPairs_].key_, first, last-first);
    pairs_[nbPairs_].key_[last-first] = 0;
    
    first = last+1;
    
    last = strchr(last, ';');
    if (last == (char*)0)
    {
      last = &ibuf[bufLen];
      debug("No more keys. Ending.");
      end = true;
    }
    memcpy(pairs_[nbPairs_].value_, first, last-first);
    pairs_[nbPairs_].value_[last-first] = 0;
    
    debug("%d : Key='%s' / Value='%s'", nbPairs_, pairs_[nbPairs_].key_, pairs_[nbPairs_].value_);
    
    first = last+1;
    nbPairs_++;
  }
  
  debug("KVBuffer::initialize : (nbPairs=%d) End OK", nbPairs_);
  
  return true;
}

int KVBuffer::getIntValue(const char* key) const
{
  const char* value = getValue(key);
  if (value == 0)
  {
    return -1;
  }
  return atoi(value);
}

const char* KVBuffer::getValue(const char* key) const
{
  for (int i=0;i<nbPairs_;i++)
  {
    if (strcmp(pairs_[i].key_, key) == 0)
    {
      debug("KVBuffer::getValue: key='%s' / value='%s'", key, pairs_[i].value_);
      return pairs_[i].value_;
    }
    
    //debug("Pair doesn't match : try='%s' / key='%s'", key, pairs_[i].key_);
  }
  
  debug("KVBuffer::getValue: key='%s' / value=<not found>", key);
  return 0;
}

bool KVBuffer::addKeyValuePair(const char* key_value)
{
  debug("KVBuffer::addKeyValuePair: key_value: %s", key_value);
#ifdef WIN32
  const char* value = strchr(key_value, '=');
#else
  const char* value = index(key_value, '=');
#endif
  if (value != 0)
  {
    char key[64];
    int len = value-key_value;
    strncpy(key, key_value, len);
    key[len] = 0;
    value++;
    debug("KVBuffer::addKeyValuePair: key=%s", key);
    debug("KVBuffer::addKeyValuePair: value=%s", value);
    return addKeyValue(key, value);
  }
  debug("KVBuffer::addKeyValuePair: Wrong key-value pair : %s", key_value);
  return false;
}
bool KVBuffer::addKeyValue(const char* key, const char* value)
{
  if (nbPairs_ >= MAX_NB_PAIRS-1)
  {
    debug("KVBuffer: MAX_NB_PAIRS=%d overloaded! Aborting", MAX_NB_PAIRS);
    return false;
  }
  
  int len = strlen(value);
  if (len > MAX_VALUE_LEN)
  {
    ((char*)value)[MAX_VALUE_LEN-1] = 0;
  }
  len = strlen(key);
  if (len > MAX_KEY_LEN)
  {
    ((char*)key)[MAX_KEY_LEN-1] = 0;
  }
  
  debug("KVBuffer::addKeyValue : Pair=%d: key='%s' / value='%s'", nbPairs_, key, value);
  strncpy(pairs_[nbPairs_].key_, key, MAX_KEY_LEN);
  strncpy(pairs_[nbPairs_].value_, value, MAX_VALUE_LEN);
  nbPairs_++;
  return true;
}
bool KVBuffer::addKeyIntValue(const char* key, int value)
{
  char str[32];
  sprintf(str, "%d", value);
  return addKeyValue(key, str);
}

int KVBuffer::getString(char* obuf, int obuf_len) const
{
  static char small[1024];
  static char* tmp = 0;
  static int real_lentmp = 2048;
  
  if (tmp == 0)
  {
    tmp = new char[real_lentmp];
  }
  
  tmp[0] = 0;
  small[0] = 0;
  for (int i=0;i<nbPairs_;i++)
  {
    if (i > 0)
    {
      strcat(tmp, ";");
    }
    snprintf(small, 1024, "%s=%s", pairs_[i].key_, pairs_[i].value_);
    
    int len_tmp = strlen(tmp);
    int len_small = strlen(small);
    if ( (len_tmp+len_small) > real_lentmp)
    {
      real_lentmp = real_lentmp * 2 + len_small;
      char* t = new char[real_lentmp];
      
      strncpy(t, tmp, len_tmp);
      delete [] tmp;
      tmp = new char[real_lentmp];
      strncpy(tmp, t, len_tmp);
    }
    
    strncat(tmp, small, real_lentmp-len_tmp );
  }
  int len_tmp = strlen(tmp);
  //strncpy(obuf, tmp, obuf_len);
  
  int nbytes = strlen(tmp) + 8;
  int size_of_nbytes = 0;
  if (nbytes < 9)
  {
    nbytes += 1;
  }
  else if (nbytes < 98)
  {
    nbytes += 2;
  }
  else if (nbytes < 997)
  {
    nbytes += 3;
  }
  else
  {
    nbytes += 4;
  }
  
  sprintf(small, "nbytes=%d", nbytes);
  
  snprintf(obuf, obuf_len, "%s;%s", small, tmp);
  return strlen(obuf);
}
