
#ifndef COMMON_H
#define COMMON_H


#include <windows.h>


// name of the executable
#define SZAPPNAME				"NSServ"
// internal name of the service
#define SZSERVICENAME			"NetSaintService"
// displayed name of the service
#define SZSERVICEDISPLAYNAME	"NetSaint Service"
// list of service dependencies - "dep1\0dep2\0\0"
#define SZDEPENDENCIES			""

// name of the service manager
extern const TCHAR* SZSERVMANNAME;

// NI file name
extern const TCHAR* INIFILE;

extern const char* PROSLISTFILE;
extern const char* DEBUG_PROSLISTFILE;


#define BUF_SIZE				1000 // max  buffersize

#define MAX_PASSWD_LEN			19	 // max characters in password
#define MAXPATH				    4096 // max characters in install-path




// Only for NT
extern const TCHAR* CPULOG;
extern const TCHAR* MSGFILE;
extern const TCHAR* ERRLOG;
extern const TCHAR* SOCKLOG;

// Debug messages
#define ERROR_LEVEL     0
#define WARNING_LEVEL   1
#define INFO_LEVEL      2
#define DEBUG_LEVEL     3
#define PRIO_LEVEL      -1




// globals, needed to communicate with the main-thread
extern struct ProcessInfo	Process;
extern CRITICAL_SECTION		ProcessAccess;
extern struct CPUInfo		CPU;
extern BOOL					cpuThrdIsRunning;





//
//  FUNCTION: void PrintToFile(char* str, char* log)
//
//  PURPOSE: print text to file 
//
//  PARAMETERS:
//    str - string to print
//    log - file to write to
//
//  RETURN VALUE:
//    none
//
//
void debug(const char* format, ...);
void truncateLogFile(const char* logFile);
void printToLog(int level, const char* logFile, const char* format, ...);
//void PrintSocketError(int errCode, char* where, const char* log);

//
//  FUNCTION: GetLastErrorText
//
//  PURPOSE: copies error message text to string
//
//  PARAMETERS:
//    lpszBuf - destination buffer
//    dwSize - size of buffer
//	  dwErr - error code returned by GetLastError()
//
//  RETURN VALUE:
//    destination buffer
//
//
LPTSTR GetLastErrorText();
LPTSTR GetLastErrorText(DWORD err);
LPTSTR GetLastErrorText( LPTSTR lpszBuf, DWORD dwSize, DWORD dwErr );




extern BOOL g_Debug;
extern int g_LogLevel;
extern char* g_WinDir;					// Windows directory.  The INIFILE has to reside in this directory !!
extern char* g_InstallDir;	// Directory where we are installed


// InputBuffer
#include "../../../Common/kvbuffer.h"


#endif

