
#include "checkService.h"

#include "../../../Common/kvbuffer.h"


DWORD CheckService(const char* servName, DWORD &stat)
{
	SC_HANDLE			schService;
    SC_HANDLE			schSCManager;
	SERVICE_STATUS		ssStatus;
	DWORD				dwRet;
	DWORD				bufSize=0;
	char				*lpServiceName=0;

	stat = -1;

	// Open a handle to the SC Manager database. 
	schSCManager = OpenSCManager( NULL,				// machine (NULL == local)
								  NULL,				// database (NULL == default)
								  GENERIC_READ );	// access required
		
	if ( schSCManager ) 
	{			
		GetServiceKeyName(	schSCManager,	// handle to a service control manager database
							servName,		// the service's display name
							lpServiceName,	// buffer to receive the service name
							&bufSize );		// size of service name buffer and service name

    // OK
		lpServiceName = new char[bufSize+1];
		
		GetServiceKeyName(	schSCManager,	
							servName,		
							lpServiceName,	
							&bufSize );
		
		// Open a handle to the service.
		schService = OpenService( schSCManager,			 // SCM database 
								  lpServiceName,		 // service name
								  SERVICE_INTERROGATE ); // Requests the service to update  
														 // immediately its current status 
														 // information to the service control
														 // manager
		delete [] lpServiceName;
				
		if (schService == NULL)  
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);			
			return dwRet;						
		}
		
		// Send a interrogate control code to the service.
		if( !ControlService( schService,					// handle to service	 
							 SERVICE_CONTROL_INTERROGATE,   // control code
							 &ssStatus ))					// pointer to service status structure
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);
			CloseServiceHandle(schService);			
			return dwRet;			
		}
	}
	else
		return GetLastError();
	
	stat = ssStatus.dwCurrentState;

	CloseServiceHandle(schSCManager);
	CloseServiceHandle(schService);
	
	return ERROR_SUCCESS;
}

DWORD CheckServices(const KVBuffer& rbuf, KVBuffer& sbuf)
{	
	DWORD	index=0;
	DWORD	obufIndex=100;
	DWORD	i2=0;
	char	*servName=0;
	DWORD	stat;
	DWORD	err;
	
	bool ok = true;
  bool end = false;
  int i=0;
	while(!end)
	{
    char str[32];
    sprintf(str, "service%d", i);
    i++;
		const char* servName = rbuf.getValue(str);
    if (servName != 0)
    {
  		err = CheckService(servName, stat);
		  if(err != 0)
		  {
			  ok = false;
			  switch(err)
			  {
			  case ERROR_INVALID_NAME:
			  case ERROR_SERVICE_DOES_NOT_EXIST:
				  AddServStatToBuffer(sbuf, servName, SERV_SERVICE_DOES_NOT_EXIST);
				  break;
			  case ERROR_SERVICE_NOT_ACTIVE:
				  AddServStatToBuffer(sbuf, servName, SERV_SERVICE_NOT_ACTIVE);
				  break;
			  case ERROR_SERVICE_REQUEST_TIMEOUT:
				  AddServStatToBuffer(sbuf, servName, SERV_SERVICE_REQUEST_TIMEOUT);
				  break;
			  default:
				  AddServStatToBuffer(sbuf, servName, SERV_CHECK_ERROR);							
			  }
		  }
		  else
		  {
			  switch(stat)
			  {
			  case SERVICE_RUNNING:
			  case SERVICE_START_PENDING:
				  AddServStatToBuffer(sbuf, servName, SERV_RUNNING);
				  break;
			  case SERVICE_PAUSED :
			  case SERVICE_PAUSE_PENDING:
				  AddServStatToBuffer(sbuf, servName, SERV_PAUSED);
				  ok=false;
				  break;
			  default:
				  AddServStatToBuffer(sbuf, servName, SERV_SERVICE_NOT_ACTIVE);				
				  ok=false;
			  }
		  }
    }
    else
    {
      end = true;
    }
	}

  return true;
}

void AddServStatToBuffer(KVBuffer& buf, const char* servName, int stat)
{
	buf.addKeyIntValue(servName, stat);
}  

