
#include "cpuLoad.h"

#include "../../../Common/error.h"





bool BuildProcessList(HQUERY &hQuery, struct ProcessInfo*& prosInfoTab, int &prosCount)
{							
  char    tmp[MAX_PROCESS_NAME_LEN];
  int			res;
  DWORD		BufSize = 0;
  PDH_STATUS	err;
  LPTSTR	CounterListBuf = NULL;
  DWORD		CounterListSize = 0;
  LPTSTR	InstanceListBuf = NULL;
  DWORD		InstanceListSize = 0;
  LPSTR		ThisInstance = NULL;
  
  PDH_COUNTER_PATH_ELEMENTS CounterPathElements;
  
  CounterPathElements.szMachineName = NULL;
  CounterPathElements.szObjectName = "Process";
  CounterPathElements.szParentInstance = NULL;
  CounterPathElements.szCounterName = "% Processor Time";
  
  if(hQuery!=NULL)
  {
    PdhCloseQuery(hQuery); 
  }
  
  prosCount = 0;
  
  err = PdhOpenQuery (0, 0, &hQuery);
  
  assert( err == ERROR_SUCCESS);
  if(err != ERROR_SUCCESS) 
  {
    printToLog(ERROR_LEVEL, CPULOG, "Error in BuildProcessList: PdhOpenQuery: %s", GetLastErrorText());
    return false;
  }		
  
  
  if (!GetProcessList(prosInfoTab, prosCount))
  {
    delete [] prosInfoTab;
    prosInfoTab = 0;
    return false;
  }
  
  
  // Communicate
  for (int i=0;i<prosCount;i++)
  {
    prosInfoTab[i].instance = 0;
    
    // Check if there are more instances of this process
    int cnt = 0;
    for (int j=0;j<i+1;j++)
    {			
      res = strcmp(prosInfoTab[i].processName, prosInfoTab[j].processName);
      if( res == 0 )
      {
        cnt++;
        if (cnt > 1)
        {
          prosInfoTab[i].instance++;
        }
      }
    }
    
    // Build counter path	
    CounterPathElements.szInstanceName = prosInfoTab[i].processName;
    CounterPathElements.dwInstanceIndex = prosInfoTab[i].instance;
    
    tmp[0] = 0;
    BufSize = MAX_PROCESS_NAME_LEN;
    err = PdhMakeCounterPath( &CounterPathElements,
						tmp,
            &BufSize,
            0 );
    
    if(err != ERROR_SUCCESS) 
    {
      int			_i;
      char		errMsg[512];
      
      _i  = sprintf(errMsg, "PdhMakeCounterPath in BuildProcessList. ");
      _i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
      _i += sprintf(errMsg + _i, "Process %s will not be added to the process list", prosInfoTab[i].processName);				
      printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
      
      continue;				
    }						
    
    err = PdhAddCounter( hQuery, tmp, 0, &prosInfoTab[i].hCounter);
    
    assert( err == ERROR_SUCCESS);
    if(err != ERROR_SUCCESS) 
    {
      int			_i;
      char		errMsg[512];
      
      _i  = sprintf(errMsg, "PdhAddCounter in BuildProcessList. ");
      _i += sprintf(errMsg + _i, "Pdh message code: 0x%X ", err);
      _i += sprintf(errMsg + _i, "Could not add process %s to the process list", prosInfoTab[i].processName);				
      printToLog(ERROR_LEVEL, CPULOG, errMsg, GetLastErrorText());
      
      continue;
    }
    
  }

  delete [] prosInfoTab;
  prosInfoTab = 0;
  
  // update, necessary to make the counter value valid		
  err = PdhCollectQueryData(hQuery);	
  assert( err == ERROR_SUCCESS);		
  
  return true;
}


DWORD WINAPI CollectCpuLoad(LPVOID lpvParam) 
{	
  HQUERY		hCPUQuery = NULL; 
  HCOUNTER	hCpuCounter = NULL;	
  
  HQUERY		hProsQuery = NULL; 
  HCOUNTER	hProcessCounter = NULL;
  
  DWORD					ctrType;
  PDH_FMT_COUNTERVALUE	fmtValue;
  
  PDH_STATUS	err = ERROR_SUCCESS;
  bool ok = true;
  
  //LPTSTR		CounterListBuf = NULL;
  //DWORD		CounterListSize = 0;
  //LPTSTR		InstanceListBuf = NULL;
  //DWORD		InstanceListSize = 0;
  int			prosCount = 0;	
  bool		refreshProcesses = true;
  int			max = 0;	
  int			cnt = 0;	
  
  //char		errMsg[512];
  //int			_i;
  
  char		cpuTotCounterPath[256];
  DWORD		version;
  DWORD		majorVersion;
  DWORD		minorVersion;
  
  HANDLE		hCurPros;
  
  // tell our parent that we are running
  cpuThrdIsRunning = true;
  
  // get Windows-version
  version = GetVersion();
  majorVersion = (DWORD) (LOBYTE(LOWORD(version)));
  minorVersion = (DWORD) (HIBYTE(LOWORD(version)));
  
  if (majorVersion < 5) // NT4
    strcpy(cpuTotCounterPath,"\\System\\% Total Processor Time");
  else // W2K
    strcpy(cpuTotCounterPath,"\\Processor(_Total)\\% Processor Time");	
  
  for(int i=0; i < CPULoadArraySize; i++)
  {
    CPU.CpuLoadLast_15_Min[i] = 0;
  }
  
  CPU.CPULoadArrayIndex = 0;	
  
  Process.cpuLoad = 0;	
  
  struct ProcessInfo* prosInfoTab = 0;
  
  err = PdhOpenQuery (0, 0, &hCPUQuery);
  
  assert( err == ERROR_SUCCESS);
  if(err != ERROR_SUCCESS) 
  {		
    printToLog(ERROR_LEVEL, CPULOG, "PdhOpenQuery in CollectCpuLoad: %s", GetLastErrorText());
    goto cleanup;
  }	
  
  // Counter for TOTAL CPU-load
  err = PdhAddCounter( hCPUQuery, cpuTotCounterPath, 0,  &hCpuCounter);
  
  assert( err == ERROR_SUCCESS);
  if(err != ERROR_SUCCESS) 
  {
    printToLog(ERROR_LEVEL, CPULOG, "PdhAddCounter in CollectCpuLoad: %s", GetLastErrorText());
    goto cleanup;
  }	
  
  // Counter for number of processes running on the system
  err = PdhAddCounter( hCPUQuery, "\\Objects\\Processes", 0,  &hProcessCounter) ;
  
  assert( err == ERROR_SUCCESS);
  if(err != ERROR_SUCCESS) 
  {
    printToLog(ERROR_LEVEL, CPULOG, "PdhAddCounter in CollectCpuLoad: %s", GetLastErrorText());
    goto cleanup;
  }

  // FIXME
  while(1)
  {
    Sleep(1000);
  }
  // FIXME
  
  while(1) 
  {
    // collect data: cpu-load and number of processes
    err = PdhCollectQueryData(hCPUQuery);
    
    assert( err == ERROR_SUCCESS);
    if(err != ERROR_SUCCESS) 
    {
      printToLog(ERROR_LEVEL, CPULOG, "PdhCollectQueryData in CollectCpuLoad: %s", GetLastErrorText());
      break;
    }		
    
    // number of processes running on the system
    fmtValue.longValue = 0;
    err = PdhGetFormattedCounterValue( hProcessCounter, PDH_FMT_LONG, &ctrType, &fmtValue);		
    
    assert( err == ERROR_SUCCESS);
    if(err != ERROR_SUCCESS) 
    {
      printToLog(ERROR_LEVEL, CPULOG, "PdhGetFormattedCounterValue in CollectCpuLoad: %s", GetLastErrorText());
      break;
    }		
    
    // if the process count changes, build the process list again
    if( refreshProcesses )
    {			
      prosCount = fmtValue.longValue;
      delete [] prosInfoTab;
      prosInfoTab = 0;
      
      ok = BuildProcessList(hProsQuery, prosInfoTab, prosCount);
      
      assert(ok);
      if(!ok)
      {
        printToLog(ERROR_LEVEL, CPULOG, "Failed to build process list: %s", GetLastErrorText());
        break;
      }			
      
      refreshProcesses = false;
      
      // remove as many pages as possible from the working set			
      hCurPros = GetCurrentProcess(); 
      const char* error = GetLastErrorText();
      SetProcessWorkingSetSize( hCurPros, 0xFFFFFFFF , 0xFFFFFFFF );
      CloseHandle(hCurPros);
    }
    
    // Collect CPU-data for all the processes on the system
    err = PdhCollectQueryData(hProsQuery);
    
    assert( err == ERROR_SUCCESS);
    if(err != ERROR_SUCCESS) 
    {
      printToLog(ERROR_LEVEL, CPULOG, "PdhCollectQueryData in CollectCpuLoad: %s", GetLastErrorText());
      break;
    }		
    
    err = PdhGetFormattedCounterValue( hCpuCounter, PDH_FMT_LONG, &ctrType, &fmtValue);
    
    assert( err == ERROR_SUCCESS);
    if(err != ERROR_SUCCESS) 
    {
      printToLog(ERROR_LEVEL, CPULOG, "PdhGetFormattedCounterValue in CollectCpuLoad: %s", GetLastErrorText());
      break;
    }					
    
    // Update the arrary
    EnterCriticalSection(&ProcessAccess);
    
    CPU.CpuLoadLast_15_Min[CPU.CPULoadArrayIndex] = (short) fmtValue.longValue;
    CPU.CPULoadArrayIndex = (CPU.CPULoadArrayIndex + 1) % CPULoadArraySize;
    
    // Find the process with highest CPU-load
    max = 0;
    prosInfoTab[0].cpuLoad = max;
    Process = prosInfoTab[0];		
    
    for(int i=0; i < prosCount; i++) 
    {			
      err = PdhGetFormattedCounterValue( prosInfoTab[i].hCounter, PDH_FMT_LONG, &ctrType, &fmtValue);
      if( err != ERROR_SUCCESS )
      {
        //printToLog(ERROR_LEVEL, CPULOG, "PdhGetFormattedCounterValue in CollectCpuLoad: %s", GetLastErrorText());
        
        // try to build the process list again 
        refreshProcesses = true;
        continue;
      }	
      
      prosInfoTab[i].cpuLoad = (short) fmtValue.longValue;
      
      
      if( fmtValue.longValue > max) 
      {
        max = fmtValue.longValue;								
        Process = prosInfoTab[i];				
      }			
    }
    
    LeaveCriticalSection(&ProcessAccess);
    
    Sleep(SAMPLE_INTERVAL);	
  }

cleanup:
  // tell that we are stopped
  cpuThrdIsRunning = false;

  delete [] prosInfoTab;
  prosInfoTab = 0;

  if(hProsQuery!=NULL)
  {
    PdhCloseQuery(hProsQuery);
  }
  if(hCPUQuery!=NULL)
  {
    PdhCloseQuery(hCPUQuery);
  }

  return err;
}

bool GetProcessList(struct ProcessInfo*& prosInfoTab, int& prosCount)
{
	PDH_STATUS	err;
	LPTSTR		CounterListBuf = NULL;
	DWORD		CounterListSize = 0;
	LPTSTR		InstanceListBuf = NULL;
	DWORD		InstanceListSize = 0;
	LPSTR		ThisInstance = NULL;
	HQUERY		hQuery;

	prosCount = 0;

  delete [] prosInfoTab;
  prosInfoTab = 0;
	
  err = PdhOpenQuery (NULL, 0, &hQuery);
	assert( err == ERROR_SUCCESS);
	
	// Get bufferspace needed to build the process-array
	err = PdhEnumObjectItems( NULL,
							  NULL,
							  TEXT("Process"),
							  CounterListBuf,
							  &CounterListSize,
							  InstanceListBuf,		
							  &InstanceListSize,
							  PERF_DETAIL_WIZARD,
							  0);
	assert( err == ERROR_SUCCESS);

	// Allocate memory
	CounterListBuf  = new TCHAR[CounterListSize];
  InstanceListBuf = new TCHAR[InstanceListSize];

	// Call PdhEnumObjectItems again and build an array 
	// of all processes running on the system
	if(CounterListBuf != NULL && InstanceListBuf  != NULL) 
  {
		err = PdhEnumObjectItems( NULL,
							  NULL,
							  TEXT("Process"),
							  CounterListBuf,
							  &CounterListSize,
							  InstanceListBuf,		
							  &InstanceListSize,
							  PERF_DETAIL_WIZARD,
							  0);
		assert( err == ERROR_SUCCESS);
		
		BOOL res = true;

    // First time, just to get the number of procs
		for (ThisInstance = InstanceListBuf; *ThisInstance != 0 ; ThisInstance += lstrlen(ThisInstance) +1 ) 
		{					
			if( strcmp(ThisInstance,"Idle") != 0 && strcmp(ThisInstance,"_Total") != 0 )
      {
				prosCount++;
			}	
		}

    // Now we make the buffer
    prosInfoTab = new struct ProcessInfo[prosCount];

    // And we copy it there
    int index = 0;
		for (ThisInstance = InstanceListBuf; *ThisInstance != 0 ; ThisInstance += lstrlen(ThisInstance) +1 ) 
		{					
			if( strcmp(ThisInstance,"Idle") != 0 && strcmp(ThisInstance,"_Total") != 0 )
      {
        strncpy(prosInfoTab[index].processName, ThisInstance, MAX_PROCESS_NAME_LEN);
        index++;
			}	
		}
    if (index != prosCount)
    {
    	delete [] CounterListBuf;
      delete [] InstanceListBuf;
      delete [] prosInfoTab;
      prosInfoTab = 0;
      assert(0);
      return false;
    }
	}
  else
  {
    delete [] CounterListBuf;
    delete [] InstanceListBuf;
    delete [] prosInfoTab;
    prosInfoTab = 0;
    return false;
  }
	
	PdhCloseQuery(hQuery);
	
  delete [] CounterListBuf;
  delete [] InstanceListBuf;

	return true;
}

