

#include "eventLog.h"

//////////////////////////////////////////////////////////////////////////////


using namespace std;
typedef list<SourceData> SourceDataList;


//
//  FUNCTION: int GetNewRecords( char* logName, 
//								 SourceDataList	&srcList,				   
//								 DWORD nSec )
//
//  PURPOSE: Get all logrecords added to the specified eventlog file 
//			 during the last N seconds (inclusive description strings)
//
//  PARAMETERS:
//		logName - name of the logfile to open
//		srcList - list which new eventlogrecords will be added to 
//		nSec - number of seconds to "look back" in the eventlog
//		nErr - number
//		
//  RETURN VALUES:
//		number of records added to the list if the funtion returned successfully		
//		-1 if the function did not succeed
//
int GetNewRecords( char*			logName, 
                  SourceDataList	&srcList,				   
                  DWORD			nSec );


//
//  FUNCTION: BuildEventLogBuffer( SourceDataList	&srcList, 
//								   EventLogLimits*	lim,
//								   char*			buffer,
//								   DWORD			bufSize)
//
//  PURPOSE: build a buffer of eventlog records which has exceeded any of
//			 the limits specified.  This is the buffer which is sent to the client
//
//  PARAMETERS:
//		srcList - list which contains eventlog records
//		lim	- limits which the eventlog records cannot exceed
//		buffer - buffer of records that has exceeded any of the limits given				 
//
//  RETURN VALUES:
//		none
//
//	COMMENT:
//	the buffer has the following format:
// --------------------------------------------------------------------------------------------
// |nbytes=N;cname=COMPUTER;stat=STATUS;src=SOURCENAME;id=ID;type=TYPE;cnt=COUNT;msg=MESSAGE;;|
// --------------------------------------------------------------------------------------------
//				
// nbytes - number of bytes in the buffer
// cname - name of this computer
// stat - status of the eventlog (based on the limits)
//		  could be 'error', 'warning' or 'ok'
// src - name of the source which has has broken any of the limits
// id - the event id of the record
// type - message type (information, error, warning, auditFailure or auditSuccess)
// cnt - number of messages of this message id
// msg - the description string of this record
//
// the buffer can contain multiple records if neccessary
//
void BuildEventLogBuffer( SourceDataList& srcList, 
                         EventLogLimits* lim,
                         KVBuffer& buffer);

//
//  FUNCTION: GetBytesNeededForEventLogBuffer
//
//  PURPOSE: Compute the number of bytes needed to build a
//			 buffer of eventlog records
//
//  PARAMETERS:
//		srcList - list which contains eventlog records
//		lim	- limits which the eventlog records cannot exceed
//		nBytesNeeded - conatins the number of bytes needed when the
//					   function has returned
//  RETURN VALUES:
//		none		
//	
/*
void GetBytesNeededForEventLogBuffer( const SourceDataList	&srcList,									 
EventLogLimits*		lim,
int					&nBytesNeeded);
*/


//////////////////////////////////////////////////////////////////////////////


void AddEventSource(char *installDir)
{
  HKEY hk; 
  DWORD dwData; 
  char*	szBuf;
  int		bufSize;	
  char*	regKey = "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\";
  
  if(strlen(regKey) > strlen(installDir))
  {
    bufSize =   strlen(regKey) + strlen(SZAPPNAME);	
  }
  else
  {
    bufSize =   strlen(installDir+1) + strlen(SZAPPNAME);	
  }
  
  szBuf = new char[bufSize+128];
  
  strcpy(szBuf, regKey);	
  strcat(szBuf, SZAPPNAME);
  
  // Add your source name as a subkey under the Application 
  // key in the EventLog registry key. 
  
  if (RegCreateKey(HKEY_LOCAL_MACHINE, szBuf, &hk))
  {
    printToLog(ERROR_LEVEL, ERRFILE, "Could not create the registry key");
    delete [] szBuf;
    szBuf = 0;
    return;
  }     
  
  // Set the name of the message file
  
  strcpy(szBuf, installDir);
  strcat(szBuf, "\\");
  strcat(szBuf, SZAPPNAME);
  strcat(szBuf, ".exe");
  
  // Add the name to the EventMessageFile subkey. 
  
  if (RegSetValueEx(hk,             // subkey handle 
    "EventMessageFile",       // value name 
    0,                        // must be zero 
    REG_EXPAND_SZ,            // value type 
    (LPBYTE) szBuf,      // pointer to value data 
    strlen(szBuf) + 1))  // length of value data 
  {
    printToLog(ERROR_LEVEL, ERRFILE, "Could not set the event message file");
    RegCloseKey(hk);
    delete [] szBuf;
    szBuf = 0;
    return;	
  }       
  
  // Set the supported event types in the TypesSupported subkey. 
  
  dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | 
    EVENTLOG_INFORMATION_TYPE; 
  
  if (RegSetValueEx(hk,      // subkey handle 
    "TypesSupported",  // value name 
    0,                 // must be zero 
    REG_DWORD,         // value type 
    (LPBYTE) &dwData,  // pointer to value data 
    sizeof(DWORD)))    // length of value data 
  {
    printToLog(ERROR_LEVEL, ERRFILE, "ERR: Could not set the supported types");
  }	
  
  delete [] szBuf;
  szBuf = 0;
  
  RegCloseKey(hk);
} 




bool GetDescriptionString(char				*sourceName,
                          char				*logName, 
                          EVENTLOGRECORD	*record, 
                          char				*descStr, 
                          int				&bufSize)
{
  int			nBytes=0;
  int			totBytes=0;
  bool		ret=true;
  int			res;
  HKEY		hRegKey = NULL;	
  char		*pSubKey=NULL;
  int			strLen;
  char	    *szBuf=NULL;	// buffer to contain messagefile-name
  DWORD		szBufSize;	
  char		*str1 = "SYSTEM\\CurrentControlSet\\Services\\EventLog\\";	
  HMODULE		hModule = NULL;	
  LPTSTR		lpszTemp = NULL;
  DWORD		dwFormatFlags;
  char		**msgTable = NULL;
  int			offset;
  int			i;
  
  strLen = strlen(sourceName) + strlen(logName) + strlen(str1);
  
  pSubKey = new char[strLen+10];
  
  strcpy(pSubKey, str1);
  strcat(pSubKey, logName);
  strcat(pSubKey, "\\");
  strcat(pSubKey, sourceName);
  
  res = RegOpenKey( HKEY_LOCAL_MACHINE,	// handle to open key
    pSubKey,				// address of name of subkey to open
    &hRegKey );			// address of handle to open key
  
  
  if( res != ERROR_SUCCESS) 
  {
    ret = false;
    goto cleanup;
  }
  
  // get required bufferspace
  res =  RegQueryValueEx( hRegKey,			 // handle to key to query
    "EventMessageFile",  // address of name of value to query
    0,					 // reserved
    NULL,				 // address of buffer for value type
    NULL,			  	 // address of data buffer
    &szBufSize);		 // address of data buffer size
  
  if( res != ERROR_SUCCESS) 
  {
    ret = false;
    goto cleanup;
  }
  
  szBuf = new char[szBufSize];
  
  // get name of messagefile
  res =  RegQueryValueEx( hRegKey,			 // handle to key to query
    "EventMessageFile",  // address of name of value to query
    0,					 // reserved
    NULL,				 // address of buffer for value type
    (LPBYTE) szBuf,      // address of data buffer
    &szBufSize);		 // address of data buffer size
  
  if( res != ERROR_SUCCESS) 
  {
    ret = false;
    goto cleanup;
  }
  
  if( szBuf[0]=='%' ) // "%SystemRoot%"
  {
    char buf[256];		
    char *tmp;
    GetWindowsDirectory(buf, 256);
    
    tmp = new char[strlen(buf)+strlen(szBuf)];
    
    strcpy(tmp, buf);
    strcat(tmp, &szBuf[12]);
    
    delete [] szBuf;
    szBuf = new char[strlen(tmp)+1];
    strcpy(szBuf, tmp);
    
    delete [] tmp;
    tmp = 0;
  }
  
  // open the messagefile	
  hModule = LoadLibraryEx(szBuf, NULL, LOAD_LIBRARY_AS_DATAFILE);	
  
  DWORD error;
  error = GetLastError();
  
  
  if(hModule==NULL) 
  {		
    ret = false;
    delete [] msgTable;
    msgTable = new char*[1];
    msgTable[0] = " ";
    bufSize = 1;
    goto cleanup;
  }
  
  dwFormatFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER |						  						  
    FORMAT_MESSAGE_FROM_HMODULE |
    FORMAT_MESSAGE_ARGUMENT_ARRAY;
  
  
  delete [] msgTable;
  msgTable = new char*[record->NumStrings];
  offset=0;
  
  // insert the insertion strings from the eventrecord into msgTable
  for(i=0; i<record->NumStrings; i++)	
  {
    nBytes = strlen((char*) ((LPBYTE) record + record->StringOffset + offset));
    nBytes++;		
    totBytes+=nBytes;		
    msgTable[i] = new char[nBytes];
    strncpy(msgTable[i], (char*) ((LPBYTE) record + record->StringOffset + offset), nBytes);		
    offset += strlen(msgTable[i])+1;
  }
  
  // get the description string
  res = FormatMessage( dwFormatFlags,		 // source and processing options
    hModule,			 // pointer to  message source
    record->EventID,	 // requested message identifier
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // language identifier for requested message
    (LPTSTR)&lpszTemp,	 // pointer to message buffer
    0,					 // maximum size of message buffer
    msgTable );
  
  if ( res == NULL || bufSize < strlen(lpszTemp) )
  {
    bufSize = strlen(lpszTemp);
    ret = false;
    goto cleanup;
  }		
  
  if(lpszTemp[strlen(lpszTemp)-2]==13)	// ascii 13 = carriage return
    lpszTemp[strlen(lpszTemp)-2]='\0';
  
  strcpy(descStr, lpszTemp);	
  
cleanup:	
  if(lpszTemp!=NULL)
    LocalFree((HLOCAL) lpszTemp );	
  if(hModule!=NULL)
    FreeLibrary(hModule);
  if(hRegKey!=NULL)
    CloseHandle(hRegKey);	
  
  delete [] pSubKey;
  pSubKey = 0;

  delete [] szBuf;
  szBuf = 0;

  if(msgTable != NULL)// && nBytes > 0)
  {
    for(i=0;i<record->NumStrings;i++)
    {
      delete [] msgTable[i];
    }
    delete [] msgTable;
  }
  msgTable = 0;
  
  return ret;
}

int GetNewRecords( char*			logName, 
                  SourceDataList	&srcList,				   
                  DWORD			nSec )
                  
{	
  SourceDataList::iterator it;
  
  HANDLE			h;
  EVENTLOGRECORD	*pevlr; 
  BYTE			bBuffer[1000]; 
  DWORD			dwRead, dwNeeded, dwThisRecord = 0; 
  bool			firstTime = true;
  bool			stop = false;
  bool			listUpdated=false;	
  int				nNewRecords = 0;  
  int				bytesRequired=0;
  bool			res;	
  SourceData		srcData;    
  srcData.message = NULL;	
  int				nListElements=0;
  DWORD			dwTime;
  
  h = OpenEventLog( NULL, logName);   // source name 
  if (h == NULL)
  {
    return -1;
  }
  
  pevlr = (EVENTLOGRECORD *) &bBuffer;

  time( (time_t*) &dwTime );
  dwTime -= nSec;

  while (ReadEventLog( h,							// event log handle 
    EVENTLOG_FORWARDS_READ |	// reads forward 
    EVENTLOG_SEQUENTIAL_READ,	// sequential read 
    0,							// ignored for sequential reads 
    pevlr,						// pointer to buffer 
    1000,						// size of buffer 
    &dwRead,					// number of bytes read 
    &dwNeeded)					// bytes in next record 
    && !stop )
  {
    while (dwRead > 0)
    {
      // Only items that are new enough
      if( dwTime <= pevlr->TimeGenerated)
      {				
        strncpy(srcData.sourcName, (LPSTR) ((LPBYTE) pevlr + sizeof(EVENTLOGRECORD)), 255);
        srcData.EventType = pevlr->EventType;
        srcData.eventID = pevlr->EventID;				
        srcData.genTime = pevlr->TimeGenerated;
        
        // run through the list and check if the combination
        // sourceName - eventId already exist
        for (it = srcList.begin(); it != srcList.end(); ++it)
        {
          if( strcmp(it->sourcName, srcData.sourcName) == 0
            && it->eventID == srcData.eventID )
          {					
            switch(srcData.EventType)
            {
            case EVENTLOG_ERROR_TYPE:							
              it->nErr++;							
              break;
            case EVENTLOG_WARNING_TYPE:			
              it->nWarn++;
              break;
            case EVENTLOG_INFORMATION_TYPE:				
              it->nInf++;					
              break;
            case EVENTLOG_AUDIT_SUCCESS:			
              it->nAudS++;
              break;
            case EVENTLOG_AUDIT_FAILURE:			
              it->nAudF++;
              break;						
            }
            
            it->nMessages++;							
            listUpdated = true;											
          }
        }				
        
        // A new message with a new id has been found
        if(!listUpdated) 
        {
          
          srcData.nErr = srcData.nWarn = srcData.nInf = 0;
          srcData.nAudF = srcData.nAudS = 0;								
          
          switch(srcData.EventType)
          {
          case EVENTLOG_ERROR_TYPE:
            srcData.nErr++;
            break;
          case EVENTLOG_WARNING_TYPE:			
            srcData.nWarn++;
            break;
          case EVENTLOG_INFORMATION_TYPE:				
            srcData.nInf++;
            break;
          case EVENTLOG_AUDIT_SUCCESS:			
            srcData.nAudS++;
            break;
          case EVENTLOG_AUDIT_FAILURE:			
            srcData.nAudF++;
            break;			
          }
          
          srcData.nMessages = 1;
          srcList.insert (srcList.begin(), srcData);
          it = srcList.begin();
          
          bytesRequired = 0;
          
          // Get number of bytes required for the buffer
          GetDescriptionString((char*) ((LPBYTE) pevlr + sizeof(EVENTLOGRECORD)), logName, pevlr, it->message, bytesRequired);
          it->message = new char[bytesRequired];				
          
          // Get the description string
          char* msg = it->message;
          res = GetDescriptionString((char*) ((LPBYTE) pevlr + sizeof(EVENTLOGRECORD)), logName, pevlr, it->message, bytesRequired);
          nListElements++;
          
        }
        
        listUpdated = false;				
        nNewRecords++;
      }
      else 
      {
        stop=true;
        break;
      }			
      
      dwRead -= pevlr->Length;
      pevlr = (EVENTLOGRECORD *) ((LPBYTE) pevlr + pevlr->Length);
    } 
    
    pevlr = (EVENTLOGRECORD *) &bBuffer;
    // salut
  }	
  
  CloseEventLog(h);
  return nNewRecords;
}


void BuildEventLogBuffer( SourceDataList	&srcList, 
                         EventLogLimits*	lim,
                         KVBuffer& sbuf )
{
  SourceDataList::iterator it;
  
  int			nBytesNeeded=0;
  char		status;	
  DWORD		computerNameSize=256;	
  char		computerName[256];
  char		tmp[20];
  bool		err=false;
  
  bool		ok=true;
  char		stat[15];	
  
  // get gomputer Name	
  GetComputerName( computerName,		// address of name buffer
    &computerNameSize  // address of size of name buffer
    );
  
  sbuf.addKeyValue("cname", computerName);
  
  // check the status
  strcpy(stat, "ok");
  
  for (it = srcList.end(); it != srcList.begin(); --it) 
  {
    status = CheckEventLogStat(it->EventType, it->nMessages, lim);	
    
    if(status == 'e')
    {
      strcpy(stat, "error");
      break;
    }
    else if(status == 'w')
    {
      strcpy(stat, "warning");
    }
  }	
  
  sbuf.addKeyValue("stat", stat);
  if( strcmp(stat, "ok") == 0 ) 
  {
    return;
  }
  
  // find application(s) which has broken any of the limits
  int index = 0;
  for (it = srcList.begin(); it != srcList.end(); ++it) 
  {
    if( CheckEventLogStat(it->EventType, it->nMessages, lim) != 'o')
    {						
      index++;
      char str[64];
      
      // sourcename
      _snprintf(str, 64, "src%d", index);
      sbuf.addKeyValue(str, it->sourcName);
      // generated time
      _snprintf(str, 64, "gentime%d", index);
      const char* gentime = ctime(&it->genTime);
      int min = time(0) - it->genTime;
      char str2[64];
      _snprintf(str2, 64, "%s (%d min old)", gentime, min);
      sbuf.addKeyValue(str, str2);
      // eventID
      _snprintf(str, 64, "id%d", index);
      sbuf.addKeyValue(str, _ultoa( (DWORD) (LOWORD(it->eventID)), tmp, 10) );
      // eventType
      _snprintf(str, 64, "type%d", index);
      sbuf.addKeyValue(str, _ultoa( it->EventType, tmp, 10) );
      // message count
      _snprintf(str, 64, "cnt%d", index);
      sbuf.addKeyValue(str, _ultoa( it->nMessages, tmp, 10) );
      // message			
      _snprintf(str, 64, "msg%d", index);
      sbuf.addKeyValue(str, it->message );
      int z=0; // DEBUG
    }
  }	
}

bool CheckEventLog(const KVBuffer& rbuf, KVBuffer& sbuf)
{
  int				nNewRecords=0;
  int				applog;
  int				seclog;
  int				syslog;		
  int				nBytesNeeded=0;	
  int				period;	
  int				index=0;
  bool			ret=true;
  EventLogLimits	limits;
  
  SourceDataList srcDataList;
  SourceDataList::iterator it;	
  
  period = rbuf.getIntValue("period");
  applog = rbuf.getIntValue("applog");
  seclog = rbuf.getIntValue("seclog");
  syslog = rbuf.getIntValue("syslog");
  
  limits.wAudF = rbuf.getIntValue("wAudF");
  limits.eAudF = rbuf.getIntValue("eAudF");
  limits.wAudS = rbuf.getIntValue("wAudS");
  limits.eAudS = rbuf.getIntValue("eAudS");
  limits.wErr  = rbuf.getIntValue("wErr");
  limits.eErr  = rbuf.getIntValue("eErr");
  limits.wWarn = rbuf.getIntValue("wWarn");
  limits.eWarn = rbuf.getIntValue("eWarn");
  limits.wInf  = rbuf.getIntValue("wInf");
  limits.eInf  = rbuf.getIntValue("eInf");
  
  // get number of elements needed for scrDataArr	
  if(applog) 
  {		
    GetNewRecords( "Application", 
      srcDataList,						 
      period*60 );
  }
  
  if(seclog)
  {		
    GetNewRecords( "Security", 
      srcDataList,						
      period*60 );		
  }	
  
  if(syslog)
  {		
    GetNewRecords( "System", 
      srcDataList,						
      period*60 );		
  }
  
  BuildEventLogBuffer( srcDataList, &limits, sbuf);
  
  for (it = srcDataList.begin(); it != srcDataList.end(); ++it)
  {
    delete [] it->message;
  }
  
  srcDataList.clear();
  
  return ret;
}

char CheckEventLogStat(DWORD EventType, int nErr, EventLogLimits *lim) 
{
  switch(EventType)
  {
  case EVENTLOG_ERROR_TYPE:
    {
      if(lim->wErr == 0 || nErr < lim->wErr )
      {
        return 'o';
      }
      if(nErr > lim->eErr)
      {
        return 'e';		
      }
      return 'w';		
    }
  case EVENTLOG_WARNING_TYPE:
    {
      if(lim->wWarn == 0 || nErr < lim->wWarn )
      {
        return 'o';
      }
      if(nErr > lim->eWarn)
      {
        return 'e';		
      }
      return 'w';		
    }
  case EVENTLOG_INFORMATION_TYPE:
    {
      if(lim->wInf == 0 || nErr < lim->wInf )
      {
        return 'o';
      }
      if(nErr > lim->eInf)
      {
        return 'e';
      }
      return 'w';
    }
  case EVENTLOG_AUDIT_SUCCESS:
    {
      if(lim->wAudS == 0 || nErr < lim->wAudS )
      {
        return 'o';
      }
      if(nErr > lim->eAudS)
      {
        return 'e';		
      }
      return 'w';		
    }
  case EVENTLOG_AUDIT_FAILURE:
    {
      if(lim->wAudF == 0 || nErr < lim->wAudF )
      {
        return 'o';
      }
      if(nErr > lim->eAudF)
      {
        return 'e';		
      }
      return 'w';	
    }
    
  default:
    {
      int z=0;
    }
    break;
  };
  
  return 'o';
}

