#ifndef eventLogH
#define eventLogH

#include "windows.h"
#include <iostream.h>
#include <assert.h>
#include "message.h"
#include <time.h>
#include "common.h"
#include "../../../Common/error.h"
#include "service.h"
#include <list>

#define ERRFILE		"errMainLog.txt"	// file to log error mesages


struct SourceData
{
	char	sourcName[256];	// sourcename of eventlog record
	DWORD	eventID;		// eventID of eventlog record
	DWORD	EventType;	// event type of eventlog record
  time_t genTime;    // gen time
	int		nMessages;	// number of messages of this event type	
	char*	message;		// description string of the the record
	int		nErr;			  // number of error messages this source has generated
	int		nWarn;			// number of warning messages this source has generated
	int		nInf;			  // number of information messages this source has generated
	int		nAudF;			// number of auditFailure messages this source has generated
	int		nAudS;			// number of auditSuccess messages this source has generated
};


struct EventLogLimits
{
	int			wAudF;	// number of auditFailure messages to result in a warning state
	int			eAudF;	// number of auditFailure messages to result in a error state
	int			wAudS;	// number of auditSuccess messages to result in a warning state
	int			eAudS;	// number of auditSuccess messages to result in a error state
	int			wErr;	// number of error messages to result in a warning state
	int			eErr;	// number of error messages to result in a error state
	int			wWarn;	// number of warning messages to result in a warning state
	int			eWarn;	// number of warning messages to result in a error state
	int			wInf;	// number of information messages to result in a warning state
	int			eInf;	// number of information messages to result in a error state
};


//
//  FUNCTION: void AddEventSource(char *installDir)
//
//  PURPOSE: Add an event source to the registry. This is neccessary for
//			 the Event Viewer to be able to map the event identifier codes 
//			 to message strings.			  
//
//  PARAMETERS:
//		installDir - directory where the messagefile is installed
//    
//  RETURN VALUES:
//		none
//	
void AddEventSource(char *installDir);


//
//  FUNCTION: void AddToMessageLog(LPSTR szMsg, WORD wType)
//
//  PURPOSE: Add a message string to the eventlog
//
//  PARAMETERS:
//		szMsg - the message string to add to the logfile
//		wType - message type
//
//  RETURN VALUES:
//		none				
//	
//void AddToMessageLog(LPSTR szMsg, WORD wType);


//
//  FUNCTION: bool CheckEventLog(char *ibuf, char *buffer, DWORD &bufSize)
//
//  PURPOSE: Check the eventlog and provide a buffer to send back
//			 to the client based on the values given in ibuf
//
//  PARAMETERS:
//		ibuf - buffer which
//    
//  RETURN VALUES:
//		none			
//	
bool CheckEventLog(const KVBuffer& rbuf, KVBuffer& sbuf);




//
//  FUNCTION: int GetEventlogData(char *ibuf, char *label
//
//  PURPOSE: return the limit for the specified label in ibuf
//
//  PARAMETERS:
//		ibuf - buffer received from client, contains the limits
//		label - specifies which limit we are dealing with
//    
//  RETURN VALUES:
//		the limt for the specified label				
//	
int GetEventlogData(char *ibuf, char *label);


//
//  FUNCTION: bool GetDescriptionString(char *sourceName,
//										char *logName, 
//										EVENTLOGRECORD *record, 
//										char* descStr, 
//										int &bufferSize)
//
//  PURPOSE: Get the description string for a specified record in
//			 the EventLog
//
//  PARAMETERS: 
//		sourceName - name of the source which generated the record
//		logName - specifies which logfile where the record is located
//		record - pointer to the record which "contains" the description string
//		descStr - contains the description string if the funtion returns successfully
//		bufferSize - size of the descStr
//
//  RETURN VALUES:
//		true - OK, the description string is copied into descStr
//		false - descStr is too small, bufferSize contains the size required				
//	
bool GetDescriptionString(char *sourceName,
						  char *logName, 
						  EVENTLOGRECORD *record, 
						  char* descStr, 
						  int &bufferSize);

//
//  FUNCTION: char CheckEventLogStat(DWORD EventType, 
//									 int nErr, 
//									 EventLogLimits *lim)
//
//  PURPOSE: Check if a record type has exceeded any of the limits given
//
//  PARAMETERS:
//		EventType - event type of this record
//		nErr - number of errors this record type has generated
//		lim - limits that must not be exceeded
//    
//  RETURN VALUES:
//		'o' - record type is ok
//		'w' - record type has caused a warning situation
//		'e' - record type has caused a error situation
//
char CheckEventLogStat(DWORD EventType, int nErr, EventLogLimits *lim);

#endif