

#include	"netutil.h"

#include "nsserv.h"


#include "../../../Common/queries.h"
#include "../../../Common/kvbuffer.h"




DWORD WINAPI TCP_Connection( LPVOID pParam )
{		
  int			res;
  int			adrLen = sizeof(SA);
  SOCKET		listenSock;
  SOCKET		comSock;
  SA_IN		myAdr;
  SA_IN		cliAdr;
  WSADATA		WSAData;
  char		*ibuf = 0;
  DWORD		msgBufSize = BUF_SIZE;
  char		*msgBuf = new char[msgBufSize];
  int			index;
  
  if((res=WSAStartup(MAKEWORD(1,1),&WSAData))!=0)
  {		
    printToLog(ERROR_LEVEL, TCPLOG, "Err: WSAStartup: %d", res);
    delete [] msgBuf;
    delete [] ibuf;
    return 0;
  }
  
  // create listening socket
  listenSock = socket(AF_INET, SOCK_STREAM, 0);
  
  myAdr.sin_family		= AF_INET;
  myAdr.sin_port			= htons(servPort);
  myAdr.sin_addr.s_addr	= INADDR_ANY;
  
  if(res=bind(listenSock, (SA*) &myAdr, adrLen)==SOCKET_ERROR)
  {
    sprintf(ibuf, "Err: bind: %d", WSAGetLastError()); 
    printToLog(ERROR_LEVEL, TCPLOG, ibuf);
    delete [] msgBuf;
    delete [] ibuf;
    return 0;
  }
  
  // start listening socket
  if(res =listen(listenSock, SOMAXCONN)==SOCKET_ERROR)
  {
    sprintf(ibuf, "Err: listen: %d", WSAGetLastError()); 
    printToLog(ERROR_LEVEL, TCPLOG, ibuf);
    delete [] msgBuf;
    delete [] ibuf;
    return 0;
  }
  
  while(1)
  {
    comSock=accept(listenSock, (SA*) &cliAdr, &adrLen);
    if(comSock==INVALID_SOCKET)
    {
      sprintf(ibuf, "Err: accept: %d", WSAGetLastError()); 
      printToLog(ERROR_LEVEL, TCPLOG, ibuf);
      delete [] msgBuf;
      delete [] ibuf;
      return 0;
    }

    printToLog(INFO_LEVEL, ERRLOG, "Received a new connection");
    
    index=0;

    delete [] ibuf;
    ibuf = Receive(comSock);
    if (ibuf != 0)	// message received
    {
      printToLog(INFO_LEVEL, ERRLOG, "Analyze packet : %s", ibuf);
    
      KVBuffer rbuf;
      KVBuffer sbuf;
      if (!rbuf.initialize(ibuf))
      {
        continue;
      }

      // analayze message
      res = AnalyzeTCPMessage(rbuf, sbuf, passwd);
      if(res != NOERR) 
      {
        // if error in message, discard it and continue listening
        printToLog(ERROR_LEVEL, TCPLOG, "Error in Analyze() : Msg not found");
        continue;
      }			
      
      printToLog(INFO_LEVEL, ERRLOG, "Sending packet");

      if(!Send(comSock, sbuf))
      {
        printToLog(ERROR_LEVEL, SOCKLOG, "Send() error : %s", strerror(errno) );
      }
    }
    
    
    if(msgBufSize > BUF_SIZE)
    {
      msgBufSize = BUF_SIZE;
      delete [] msgBuf;
      delete [] ibuf;
      ibuf = 0;
      msgBuf = new char[BUF_SIZE];
    }	
    
    // break connection
    res = closesocket(comSock);
    if(res==SOCKET_ERROR)
    {					
      printToLog(ERROR_LEVEL, TCPLOG, "Err: closesocket: %d", WSAGetLastError() );
      delete [] msgBuf;
      msgBuf = 0;
      delete [] ibuf;
      ibuf = 0;
      return 0;
    }

  }
  
  delete [] msgBuf;
  delete [] ibuf;

  return 0;
}

int AnalyzeTCPMessage(const KVBuffer& rbuf, KVBuffer& sbuf, const char* real_passwd) 
{
  
  int	err;
  int	id;	
  int	index=0;

  // get message id
  id = rbuf.getIntValue("id");

  printToLog(INFO_LEVEL, ERRLOG, "Message: id=%d", id);

  const char* passwd = rbuf.getValue("key");
  // check password
  if(!CheckPasswd(passwd, real_passwd))
  {
    return ERR_AUTHENTICATION;
  }
  
  printToLog(INFO_LEVEL, ERRLOG, "Check of password OK");

  err = NOERR;
  
  switch( id ) 
  {	
  case EVENTLOG_QUERY:	
    debug("\nEventLog query");
    if( !CheckEventLog(rbuf, sbuf) )
    {
      err = ERR_BUFFER_TOO_SMALL;	
    }
    break;

  case SERVICECHECK_QUERY:
    debug("\nCheckService query");
    if( !CheckServices(rbuf, sbuf))
    {
      return ERR_BUFFER_TOO_SMALL;		
    }
    break;
    
  case SELFUSAGE_QUERY:	
    debug("\nSelfUsage query");
    err = CheckYourSelf(rbuf, sbuf);
    break;

  case DISKUSAGE_QUERY:	
    debug("\nDiskUsage query");
    err = CheckDiskUsage(rbuf, sbuf);
    break;
    
  case MEMUSAGE_QUERY:	
    debug("\nMemUsage query");
    err = CheckMemoryUsage(rbuf, sbuf);
    break;

  case CPU_LOAD_QUERY:
    debug("\nCPU load query");
    err = GetCPULoadInfo(rbuf, sbuf);
    break;

  default:
    err = ERR_NO_SUCH_MESSAGE;
  }
  
  return err;
}

bool CheckPasswd(const char* test_passwd, const char* real_passwd)
{	
  debug("%s(size=%d) = %s(size=%d)", test_passwd, strlen(test_passwd), real_passwd, strlen(real_passwd) );
  if(strlen(real_passwd) != strlen(test_passwd))
  {
    debug("Password are not the same size!");
    return false;
  }
  
  if( strncmp(test_passwd, real_passwd, strlen(real_passwd)) != 0)
  {
    debug("Password are different");
    return false;
  }
  
  debug("passwd OK : %s", test_passwd);
  return true;
}


bool Send(SOCKET sock, const KVBuffer& sbuf)
{	
  int  n;
  int  index = 0;
  int buf_size = BUF_SIZE;
  char* obuf = new char[buf_size+1];

  int len = sbuf.getString(obuf, buf_size);
  while (len > buf_size)
  {
    buf_size = buf_size * 2;
    delete [] obuf;
    obuf = new char[buf_size];
    len = sbuf.getString(obuf, buf_size);
  }
  debug("Sending: (bytes=%d): %s", len, obuf);
      
  int nBytesToSend = strlen(obuf);
  while(nBytesToSend > 0)
  {
    n = send(sock, &obuf[index], nBytesToSend-index, 0);
    if(n == SOCKET_ERROR)
    {
      delete [] obuf;
      obuf = 0;
      printToLog(ERROR_LEVEL, TCPLOG, "Err: send: %d", WSAGetLastError() );
      return false;
    }

    nBytesToSend -= n;
    index = n;
    
    if(nBytesToSend <= 0)
    {
      delete [] obuf;
      obuf = 0;
      return true;
    }

  }
  
  delete [] obuf;
  obuf = 0;

  return true;
}

char* Receive(SOCKET sock)			   
{
  int 	index = 0; 
  int		nBytesRecv = 0;
  int 	n;        
  char  ch = '1';
  bool	ret=false;
  char	*ibuf = new char[BUF_SIZE+1];
  char 	*buffer = 0;
  
  int		nBytes;	    
  
  memset(ibuf, 0, BUF_SIZE);
  n = recv( sock, ibuf, BUF_SIZE, 0);
  ibuf[n]='\0';
  nBytesRecv = n;
  
  if(n==SOCKET_ERROR)	// connection lost
  {
    sprintf(ibuf, "Err: recv: %d", WSAGetLastError()); 
    printToLog(ERROR_LEVEL, TCPLOG, ibuf);
    ret=0;
    goto cleanup;
  }
  
  if(n==0)	// client terminated connection
  {
    shutdown(sock, 1);
    ret=0;
    goto cleanup;
  }    
  
  if( strncmp(ibuf, "nbytes=", 7) == 0)
  {
    nBytes = atoi(&ibuf[7]);
    
    if( nBytes < BUF_SIZE )
    {
      ibuf[strlen(ibuf)]='\0';  		
      return ibuf;
    }
    
    delete [] buffer;
    buffer = new char[nBytes];
    strncpy(buffer, ibuf, nBytes);
  }
  else
  {
    return ibuf;
  }
  
  
  while(1)
  {
    // tell that we are ready to receive next message
    n = send(sock, &ch, 1, 0);
    
    if(n == SOCKET_ERROR)
    {					
      sprintf(ibuf, "Err: send: %d", WSAGetLastError()); 
      printToLog(ERROR_LEVEL, TCPLOG, ibuf);
      ret=0;
      goto cleanup;
    }
    
    // wait for next message
    memset(ibuf,0, BUF_SIZE);
    //cout << "Waiting..." << endl;
    n = recv( sock, ibuf, BUF_SIZE, 0);
    
    if (n == 0)	// client terminated connection
    {
      shutdown(sock, 1);
      printToLog(ERROR_LEVEL, TCPLOG, "Client terminated connection");
      ret=0;
      goto cleanup;
    }
    
    if( n < 0 ) 
    {    	
      sprintf(ibuf, "Err: recv: %d", WSAGetLastError()); 
      printToLog(ERROR_LEVEL, TCPLOG, ibuf);
      ret=0;
      goto cleanup;
    }
    
    ibuf[n]='\0';
    nBytesRecv += n;
    strncat(buffer, ibuf, n);
    
    
    while( index < nBytesRecv && nBytesRecv < nBytes)
    {
      if( strncmp(&buffer[index], ";;", 2) == 0)
      {
        buffer[index+2]='\0';
        goto cleanup;	    		
      }
      
      index++;
      
      if(nBytes < index)
      {
        printToLog(ERROR_LEVEL, TCPLOG, "Error in Receive");
        ret=0;
        goto cleanup;
      }
    }	    
  }
  
cleanup:	
  delete [] ibuf;
  
  return buffer;
}
