/*******************************************************************
*
* NetSaintService for NT
* Copyright (c) 2000 Jan Christian Kaldestad 
*
*
* Description:
* This service allows clients to check:
*   1) processor load (average) and processes with highest cpu usage
*   2) events in the eventlog
*   3) status of a service
*
*
******************************************************************/

#include "nsserv.h"

#include "psapi.h"
#include "pure.h"

#include "../../../Common/kvbuffer.h"
#include "../../../Common/queries.h"



const TCHAR* SZSERVMANNAME      = "NetSaint Service Manager";

const TCHAR* INIFILE            = "NSServ.ini";

const char* PROSLISTFILE        = "eap.exe";
const char* DEBUG_PROSLISTFILE  = "..\\eap\\eap.exe";




// Global members
char		cliAddrStr[20];
int			stop=0;
char*		g_WinDir;					// Windows directory.  The INIFILE has to reside in this directory !!
char		installDir[MAXPATH];	// Directory where we are installed
char*   g_InstallDir = installDir;
HANDLE		hCpuLoadThrd;
DWORD		dwCpuLoadThrdId;
BOOL		cpuThrdIsRunning;
HANDLE		hCheckEventLogThrd;
DWORD		dwCheckEventLogThrdId;
HANDLE		hNotifyEvenetLogThrd;
DWORD		dwNotifyEvenetLogThrdId;
int			eventLogCheckInterval;
HANDLE		hCheckEventLogEvent;
HANDLE		hMainContinueEvent;
struct ProcessInfo		Process;	// Info about process with highest cpu-load
struct CPUInfo			CPU;		// Info about the cpu
CRITICAL_SECTION		ProcessAccess;
int			servPort;
char*		passwd = new char[MAX_PASSWD_LEN];
HANDLE		hTCPThrd;
DWORD		dwTCPThrdId;
int     memory_used_at_begin = 0;





int get_used_memory()
{
  BOOL ret;
  HANDLE hProcess = OpenProcess(PROCESS_SET_INFORMATION | PROCESS_QUERY_INFORMATION, FALSE, GetCurrentProcessId());
  PROCESS_MEMORY_COUNTERS ppsmemCounters;
  DWORD cb = sizeof(PROCESS_MEMORY_COUNTERS);
  double ratio = 0;
  
  ret = GetProcessMemoryInfo( hProcess,
    &ppsmemCounters, // buffer
    cb);
  
  if (ret)
  {
    int total = ppsmemCounters.WorkingSetSize + ppsmemCounters.PagefileUsage;
    ratio = (double)( total / (double)memory_used_at_begin);
    //printToLog(PRIO_LEVEL, ERRLOG, "We use now %d memory / Initial=%d / Ratio=%f\n", ppsmemCounters.WorkingSetSize, memory_used_at_begin, ratio);
    //new char[32]; // just DEBUG...
    return total;
  }

  return -1;
}



VOID ServiceStart(DWORD dwArgc, LPTSTR *lpszArgv)
{
  // initializing
  installDir[0] = 0;
  if( !Initialize(passwd) )
  {
    printToLog(ERROR_LEVEL, ERRLOG, "---- STOPPED ----");
    return;
  }
  
  // tell the SCM that we are started
  
  ReportStatusToSCMgr(SERVICE_RUNNING,NO_ERROR,30000);
  
  memory_used_at_begin = get_used_memory();
  
  int nb_handle = 0;
  while(!stop) // var is changed by another thread
  {		
    nb_handle++;
    static int interval = 500;
    if (nb_handle % interval == 60)
    {
      if (PurifyIsRunning())
      {
        debug("Finding new leaks...");
        PurifyNewLeaks();
        debug("...ok");
      }
    }

    if (nb_handle == 60)
    {
      memory_used_at_begin = get_used_memory();
    }

    if (nb_handle % 10000 == 0)
    {
      int now = get_used_memory();
      double ratio = (double)(now / (double)memory_used_at_begin);
      printToLog(PRIO_LEVEL, ERRLOG, "We use now %d memory / Initial=%d / Ratio=%f\n", now, memory_used_at_begin, ratio);
      continue;
    }
    
    int timed = 1000;
    //debug("No packet incoming. Waiting %dms", timed);
    Sleep(timed);
  }
  
  ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
  
  printToLog(PRIO_LEVEL, ERRLOG, "---- STOPPED ----");
  
  //closesocket(servSock);
  //WSACleanup();	
}

int GetCPULoadInfo(const KVBuffer& rbuf, KVBuffer& sbuf) 
{
  double load1, load2;
  
  DWORD	index = 0;
  int		interval1, interval2;

  interval1 = rbuf.getIntValue("i1")*60;
  interval2 = rbuf.getIntValue("i2")*60;
  
  if( (interval1 < 0) || (interval1 > CPULoadArraySize) || 
    (interval2 < 0) || (interval2 > CPULoadArraySize) )
  {
    return ERR_CPULOAD_INTERVAL;
  }
  
  // get requested information
  EnterCriticalSection(&ProcessAccess);

  load1 = CPU.ComputeAvgCPULoad(interval1);		
  load2 = CPU.ComputeAvgCPULoad(interval2);		
  
  sbuf.reset();

  sbuf.addKeyIntValue("id", CPU_LOAD_QUERY);
  sbuf.addKeyIntValue("l1", Round(load1));
  sbuf.addKeyIntValue("l2", Round(load2));
  sbuf.addKeyValue("pname", Process.processName);
  sbuf.addKeyIntValue("pload", Round(Process.cpuLoad));
  
  LeaveCriticalSection(&ProcessAccess);	
  
  return NOERR;
}


int Round(double num) 
{
  double tmp;
  tmp = num - floor(num);
  
  if(tmp < 0.5)
    return  floor(num);
  else
    return ( ceil(num));
}

/*
int CreateUDPSocket(SOCKET &servSock, int port) {
  
  struct sockaddr_in servaddr;
  WORD		wVersionRequired;
  WSADATA		wsaData;
  int err, WSAerr;
  unsigned long socketio;
  
  wVersionRequired = MAKEWORD(1,1);	
  
  err = WSAStartup(wVersionRequired,&wsaData); 
  WSAerr = WSAGetLastError();
  
  if(err != 0) 
  {
    printToLog(ERROR_LEVEL, SOCKLOG, "CreateUDPSocket : %s", GetLastErrorText(WSAerr) );	
    return ERR_SOCKETINIT;	
  }	
  
  // create uninitialized UDP socket
  servSock = socket( AF_INET, SOCK_DGRAM, 0);	
  
  memset(&servaddr, 0, sizeof(servaddr));
  
  // initialize the server-adress	
  servaddr.sin_family		 = AF_INET; 
  servaddr.sin_addr.s_addr = INADDR_ANY; 
  servaddr.sin_port		 = htons(port); 
  
  
  err = bind(servSock, (sockaddr *) &servaddr, sizeof(servaddr));
  WSAerr = WSAGetLastError();
  
  
  if (err == SOCKET_ERROR) 
  {
    printToLog(ERROR_LEVEL, SOCKLOG, "CreateUDPSocket : %s", GetLastErrorText(WSAerr) );	
    
    closesocket(servSock);
    WSACleanup();
    return ERR_BIND;
  }
  
  // make the socket non-blocking
  socketio = 1;
  ioctlsocket(servSock, FIONBIO, &socketio);
  
  return NOERR;
}
*/
int ReadInitData(char *passwd, int &port, int& logLevel, char* installDir)
{
  HANDLE		hFile;
  int			err=0;
  int			forts=0;
  int			nLoop=0;
  char		_port[5];
  int			strSize=1000;
  char*		buf;
  DWORD		dwFileSize;
  
  char		fileLoc[256];
  
  fileLoc[0]='\0';
  strncat(fileLoc, g_WinDir, 255-strlen(fileLoc));
  strncat(fileLoc, "\\", 255-strlen(fileLoc));
  strncat(fileLoc, INIFILE, 255-strlen(fileLoc));
  
  hFile = CreateFile( fileLoc, 
    GENERIC_READ,
    0,
    NULL,
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL,
    NULL );
  
  if(hFile == INVALID_HANDLE_VALUE) 
  {
    return ERR_NOINIFILE;		
  }
  
  dwFileSize = GetFileSize(hFile, NULL);
  if(dwFileSize ==0xFFFFFFFF)
    return ERR_FILESIZE; // FileSize error	
  
  // allocate  buffer
  buf = new char[dwFileSize];
  
  do 
  {
    ReadLine(buf, strSize, hFile);
    if( strcmp(buf,"key=") == 0 ) {				
      ReadLine(passwd, strSize, hFile);
      forts++;
    }
    else if( strcmp(buf,"port=") == 0) 
    {
      ReadLine(_port, strSize, hFile);
      port = atoi(_port);
      forts++;
    }
    else if( strcmp(buf,"dir=") == 0) 
    {
      ReadLine(buf, strSize, hFile);			
      strncpy(installDir, buf, MAXPATH);
      g_InstallDir = installDir;
      forts++;
    }
    else if( strcmp(buf,"loglevel=") == 0) 
    {
      ReadLine(_port, strSize, hFile);
      logLevel = atoi(_port);
      forts++;
    }
    
    nLoop++;				
    
  }	while(forts < 4 && nLoop < 1000); 
  
  truncateLogFile(CPULOG);
  truncateLogFile(MSGFILE);
  truncateLogFile(ERRLOG);
  truncateLogFile(SOCKLOG);
  
  printToLog(PRIO_LEVEL, ERRLOG, "---- STARTING ----");
  printToLog(PRIO_LEVEL, ERRLOG, "---- Reading INI file : %s ----", fileLoc);
  
  delete [] buf;
  
  if(nLoop == 1000 && forts == 0)
  {
    CloseHandle(hFile);
    return ERR_FILEFORMAT;		
  }
  
  CloseHandle(hFile);	
  return NOERR;
}

int ReadLine(char *str, int strSize, HANDLE hFile)
{
  int index=0, err;
  char ch=' ', forrige=' ';
  DWORD nRead;
  bool eof=false;
  bool br=false;	
  
  while(index < strSize) {
    forrige=ch;
    err= ReadFile( hFile,
      &ch,
      1,
      &nRead,
      NULL);
    
    str[index++]=ch;
    
    if( (err==1 && nRead==0) ) { // End of file
      str[index-1]='\0';
      return NOERR;;
    }
    
    if( ch=='\x0a' && forrige=='\x0d') {	// End of line
      str[index-2]='\0';
      return NOERR;;			
    }
    
    if( ch=='=' ) {
      str[index]='\0';			
      return NOERR;;
    }		
  }
  
  return 1; //ERROR
}


BOOL CheckThread(HANDLE hCpuLoadThrd, DWORD &dwCpuLoadThrdId) 
{	
  int err = 0;
  
  if(!cpuThrdIsRunning) 
  {	
    
    CloseHandle(hCpuLoadThrd);
    hCpuLoadThrd =	CreateThread(	NULL,
      0,
      CollectCpuLoad,
      NULL,
      0,
      &dwCpuLoadThrdId );
    err = GetLastError();
    
    // wait a little and check if the thread was successfully restarted
    
    if( !hCpuLoadThrd ) 
    {
      int i;
      char errMsg[256];
      i  = sprintf(errMsg, "CreateThread in CheckThread\n");
      i += sprintf(errMsg + i, "\tSystem error message: %d\n", err);
      i += sprintf(errMsg + i, "\tFailed to restart the cpu-load thread");
      printToLog(ERROR_LEVEL, ERRLOG, errMsg);
      return 0;
    }
    else 
    {
      printToLog(INFO_LEVEL, ERRLOG, "cpu-thread was successfuly restarted");
    }
    
  }
  
  return 1;
}

// called when the service is stopped
VOID ServiceStop() 
{
  ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);	
  
  // set the stop flag
  stop=1;
}

BOOL Initialize(char* passwd) 
{
  
  int			err;
  UINT		winDirSize;
  
  cpuThrdIsRunning = true;
  
  // tell the SCM that we are starting up...
  
  ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);		
  
  // get required buffersize
  
  winDirSize = GetWindowsDirectory(g_WinDir, 0);
  
  // allocate space for buffer
  
  g_WinDir = 0;
  g_WinDir = new char[winDirSize+20];
  
  // get the Windows directory
  
  if(GetWindowsDirectory(g_WinDir, winDirSize) == 0)
  {
    // operation failed, tell the SCM that we are stopping and exit
    
    ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
    printToLog(ERROR_LEVEL, ERRLOG, "GetWindowsDirectory() in ServiceStart()");
    printToLog(ERROR_LEVEL, ERRLOG, "---- STOPPED ----");
    return FALSE;
  }		
  
  // tell the SCM that we are still starting up...
  
  ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);
  
  // get values from INIFILE:
  //	- shared secret
  //	- portnumber
  //	- current directoy
  
  err = ReadInitData(passwd, servPort, g_LogLevel, installDir);	
  
  // add message-file to registry
  
  AddEventSource(installDir);	
  
  if( err != NOERR ) 
  {
    // operation failed, tell the SCM that we are stopping and exit
    
    ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
    printToLog(ERROR_LEVEL, ERRLOG, "ReadInitData() in ServiceStart()");
    printToLog(ERROR_LEVEL, ERRLOG, "---- STOPPED ----");
    return FALSE;	 	
  }
  
  // set current directory
  
  if( !SetCurrentDirectory(installDir) ) 
  {
    // operation failed, tell the SCM that we are stopping and exit
    
    ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
    printToLog(ERROR_LEVEL, ERRLOG, "ReadInitData() in ServiceStart()");
    return FALSE;	
  }
  
  
  // tell the SCM that we are still starting up...		
  
  ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);
  
  // tell the SCM that we are still starting up...
  
  ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);
  
  // start the TCP-thread
  
  hTCPThrd = CreateThread( NULL,
    0,
    TCP_Connection,
    NULL,
    0,
    &dwTCPThrdId );
  if( hTCPThrd == NULL ) 
  {
    // thread creation failed, tell the SCM and exit
    
    ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);		
    printToLog(ERROR_LEVEL, ERRLOG, "Create TCPThrd in ServiceStart()");
    return FALSE;	
  }
  
  
  // initialize critical section before we start the CPU-load thread
  
  InitializeCriticalSection(&ProcessAccess);		
  
  // start the CPU-load thread
  
  hCpuLoadThrd = CreateThread( NULL,
			 0,
       CollectCpuLoad,
       NULL,
       0,
       &dwCpuLoadThrdId );	
  if( hCpuLoadThrd == NULL )
  {
    // thread creation failed, tell the SCM and exit
    ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);		
    printToLog(ERROR_LEVEL, ERRLOG, "Create CpuLoadThrd in ServiceStart()");
    return FALSE;		
  }
  
  return TRUE;
}
