/*******************************************************************
*
* check_ntdisk plugin
* Copyright (c) 2000 Jan Christian Kaldestad
*
*
* Description:
* 	 Checks the processor load average on NT servers.
*	 Also finds the process with highest cpu usage.
*
******************************************************************/





#include "util.h"
#include "../../Common/kvbuffer.h"
#include "../../Common/error.h"
#include "../../Common/queries.h"



const char* PLUGIN_NAME = "check_ntdisk";


bool CheckCmdLine(int argc, char **argv, KVBuffer& buffer)
{
  if (argc != 7)
  {
    cout << "Incorrect arguments supplied\n\n";
    cout << "Usage: " << argv[0] << " <IPaddress> <port> <PERCENT|SIZE> <path> <wThresh> <cThresh>\n\n";
    cout << "Note:\n";
    cout << " <PERCENT|SIZE> = say if the limits are in PERCENT or MB" << endl;
    cout << " <path>      = full path to disk (or directory) to monitor" << endl;
    cout << " <wThresh>   = value of free space that result in a WARNING message," << endl;
    cout << " <cThresh>     = value of free space that result in a CRITICAL message," << endl;

    return false;
  }

  char path[128];
  strncpy(path, argv[4], 128);
  debug("Path : %s", path);

  char id[10];
  sprintf(id, "%d", DISKUSAGE_QUERY);

  buffer.addKeyValue("id", id);
  buffer.addKeyValue("path", path);

  return true;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuffer, KVBuffer& rbuffer, int& state, char* msg)
{
  int free_mb = rbuffer.getIntValue("free");
  int total_mb = rbuffer.getIntValue("total");
  int used_mb = total_mb - free_mb;

  int wThreshold = atoi(argv[5]);
  int cThreshold = atoi(argv[6]);

  debug("Used: %d / Total: %d / wThresh: %d / cThresh: %d", used_mb, total_mb, wThreshold, cThreshold);

  state = STATE_OK;
  const char* path = sbuffer.getValue("path");

  int size_or_percent = ( strcmp(argv[3], "SIZE") == 0 ? SIZE : PERCENT );

  if (size_or_percent == PERCENT)
  {
    int p = (int)(used_mb / (double)total_mb * 100);
    sprintf(msg, "Problem: %dMb (%d%s) of %dMb used on %s (%dMb free)", used_mb, p, "%%", total_mb, path, free_mb);
    debug("Msg: %s", msg);
    if (p > cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p > wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      sprintf(msg, "%dMb (%d%s) of %dMb used on %s (%dMb free)", used_mb, p, "%%", total_mb, path, free_mb);
    }
  }
  else if (size_or_percent == SIZE)
  {
    sprintf(msg, "Problem: %dMb of %dMb used on %s (%dMb free)", used_mb, total_mb, path, free_mb);

    int p = used_mb;
    if (p > cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p > wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      sprintf(msg, "%dMb of %dMb used on %s (%dMb free)", used_mb, total_mb, path, free_mb);
    }
  }
  else
  {
    state = STATE_UNKNOWN;
    sprintf(msg, "Unknown option !");
  }

  return true;
}
