/*******************************************************************
 *
 * check_eventlog plugin
 * Copyright (c) 2000 Jan Christian Kaldestad
 *
 *
 * Description:
 * 	 Checks the eventlog on NT servers.  If any application
 *   generates more than X events within a periode of Y minutes,
 *   this is reported as WARNING/CRITCAL status (according to limits
 *	 given to the plugin).  If a WARNING/CRITCAL situation occur,
 *   we generate a html-page which further describes the situation.
 *
 ******************************************************************/


#include "checkEventlog.h"


const char* PLUGIN_NAME = "check_nteventlog";


const char* baseDir = "/usr/share/netsaint/%s";
const char* baseHtml = "../%s";
const char* baseUrl = "eventlog/%s.html";
char url[256];



void showUsage(const char* program)
{
	cout << "Wrong arguments\n"
//             << "Usage: "<<program<<" <hostname> <port> <period=nb_min> <applog=[0|1]> <syslog=[0|1]> <seclog=[0|1]> <wAudF=nb> <eAudF=nb> <wAudS=nb> <eAudS=nb> <wErr=nb> <eErr=nb> <wWarn=nb> <eWarn=nb> <wInf=nb> <eInf=nb>"<<endl;
             << "Usage: "<<program<<" <hostname> <port> <period> <applog=[0|1]> <syslog=[0|1]> <seclog=[0|1]> <wAudF=nb> <eAudF=nb> <wAudS=nb> <eAudS=nb>"<<endl
             << "  where:" << endl
             << "     hostname : hostaname or IP address of host to check" << endl
             << "     port     : TCP port to connect to" << endl
             << "     period   : tell the time to check for message in EventLog" << endl
             << "     applog   : (def=0)  tell if you want the Application EventLog" << endl
             << "     syslog   : (def=0)  tell if you want the System EventLog" << endl
             << "     seclog   : (def=0)  tell if you want the Security (Audit) EventLog" << endl
             << "     wAudF    : (def=0)  tell the threshold of WARNING for critical status in Audit F" << endl
             << "     eAudF    : (def=0)  tell the threshold of ERROR for critical status in Audit F" << endl
             << "     wAudS    : (def=0)  tell the threshold of WARNING for critical status in Audit S" << endl
             << "     eAudS    : (def=0)  tell the threshold of ERRO for critical status in Audit S" << endl<<endl;
}




bool CheckCmdLine(int argc, char** argv, KVBuffer& sbuf)
{
    EventLogLimits 	limits;
    limits.wAudF = limits.eAudF = limits.wAudS = limits.eAudS = limits.wErr = limits.eErr = 0;
    limits.wWarn = limits.eWarn = limits.wInf = limits.eInf = 0;


    //strcpy(url, baseUrl);


    if (argc < 7)
    {
    	showUsage(argv[0]);
        return false;
    }

    sbuf.addKeyIntValue("id", EVENTLOG_QUERY);
    sbuf.addKeyValue("period", argv[3]);

    int i=0;
    for (i=4;i<argc;i++)
    {
    	sbuf.addKeyValuePair(argv[i]);
    }

    if (sbuf.getValue("wAudF") == 0)
    	sbuf.addKeyValue("wAudF", "0");

    if (sbuf.getValue("eAudF") == 0)
    	sbuf.addKeyValue("eAudF", "0");

    if (sbuf.getValue("wAudS") == 0)
    	sbuf.addKeyValue("wAudS", "0");

    if (sbuf.getValue("eAudS") == 0)
    	sbuf.addKeyValue("eAudS", "0");

    return true;
}

bool GenerateHtml(const KVBuffer& sbuf, const KVBuffer& rbuf)
{
	const char* cname = rbuf.getValue("cname");

        char tmp[256];
        char newName[256];
	snprintf(tmp, 256, baseUrl, cname);
	snprintf(newName, 256, baseDir, tmp);

        debug("Generate HTML in %s", newName);

	ofstream newFile(newName);

	// Write some html code
	// bugfix 25 may 2000 : added no-cache and expires to html-code (hallstein@usa.net)
	newFile << "<html>" << endl
	    << "<head><title>NSServ - Event Log Messages</title>" << endl
	    << "<META http-equiv=\"expires\" content=\"0\">" << endl
	    << "<META content=no-cache http-equiv=Pragma>" << endl
	    << "</head>" << endl
	    << "<body bgcolor=\"#FFFFFF\">" << endl
	    << "<h1>Eventlog messages</h1>" << endl
	    << "<pre>" << endl
	    << "<b>******** ";

	WriteDateAndTime(&newFile);

	newFile << " ********</b>" << endl;

        int period = sbuf.getIntValue("period");

        newFile << "Computer:      " << cname<<endl
                << "Period:        " << period << " minutes" << endl;

        const char* stat = rbuf.getValue("stat");
        newFile << "Status:        "<< stat<< endl << endl;


        bool stop = false;
        int index = 0;
	while(!stop)
	{
        	index++;
                debug("Treating msg#%d", index);

		sprintf(tmp, "src%d", index);
                const char* src = rbuf.getValue(tmp);
                if (src == 0)
                {
                	debug("No more messages (last=%d)", index);
                	stop = true;
                        break;
                }
		newFile << "Source:        "<< src << endl;

		sprintf(tmp, "id%d", index);
                int id = rbuf.getIntValue(tmp);
		newFile << "Event id:      "<< id << endl;

		sprintf(tmp, "gentime%d", index);
                const char* gentime = rbuf.getValue(tmp);
		newFile << "Gen time:      "<< gentime;

		sprintf(tmp, "type%d", index);
		int type = rbuf.getIntValue(tmp);
		newFile << "Event type:    ";
		switch(type)
		{
                case EVENTLOG_ERROR_TYPE:
                        newFile << "error";
                        break;
                case EVENTLOG_WARNING_TYPE:
                        newFile << "warning";
                        break;
                case EVENTLOG_INFORMATION_TYPE:
                        newFile << "information";
                        break;
                case EVENTLOG_AUDIT_SUCCESS:
                        newFile << "audit success" << endl;
                        break;
                case EVENTLOG_AUDIT_FAILURE:
                        newFile << "audit failure" << endl;
                default:
                        newFile << "<unknown: "<<type<<">" << endl;
                        break;
		};
		newFile << endl;

		sprintf(tmp, "cnt%d", index);
                int cnt = rbuf.getIntValue(tmp);
		newFile << "Message count: "<< cnt << endl;

		sprintf(tmp, "msg%d", index);
                const char* msg = rbuf.getValue(tmp);
		newFile << endl<<"Message:\n"<<msg<<endl;
		newFile << endl<<endl<<endl;
	}

	return true;
}

void WriteDateAndTime(ofstream* stream)
{

	time_t 	calTime;
	tm	   	t;

	time(&calTime);
	t = *localtime(&calTime);

	if(t.tm_mday<10)
		*stream << "0";

	*stream << t.tm_mday << ".";

	if(t.tm_mon+1<10)
		*stream << "0";

	*stream << t.tm_mon+1 << ".";
	*stream << t.tm_year + 1900 << " ";

	if(t.tm_hour<10)
		*stream << "0";

	*stream << t.tm_hour << ":";

	if(t.tm_min<10)
		*stream << "0";


	*stream << t.tm_min;
}

bool GetStringAndState(int argc, char** argv, KVBuffer& sbuf, KVBuffer& rbuf, int& state, char* msg)
{
	state = STATE_UNKNOWN;

       	const char* cname = rbuf.getValue("cname");
        char tmp[64];
	sprintf(tmp, baseUrl, cname);
	sprintf(url, baseHtml, tmp);
        sprintf(msg, "<A href=\"%s\">New messages in EventLog!</A>", url);

        debug("URL: %s", url);


        bool genHtml = false;
        const char* stat = rbuf.getValue("stat");
       	debug("Return status is '%s'", stat);
        if (strcmp(stat, "ok") == 0)
        {
        	state = STATE_OK;
	        sprintf(msg, "No new messages : OK");
                return true;
        }
        else if (strcmp(stat, "warning") == 0)
        {
	        genHtml = true;
        	state = STATE_WARNING;
        }
        else if (strcmp(stat, "error") == 0)
        {
	        genHtml = true;
        	state = STATE_CRITICAL;
        }
        else
        {
        	state = STATE_UNKNOWN;
	        sprintf(msg, "Unknown return status");
        }

	if (genHtml && !GenerateHtml(sbuf, rbuf))
        {
        	return false;
        }

	return true;
}
