/*******************************************************************
*
* check_ntdisk plugin
* Copyright (c) 2000 Jan Christian Kaldestad
*
*
* Description:
* 	 Checks the processor load average on NT servers.
*	 Also finds the process with highest cpu usage.
*
******************************************************************/





#include "util.h"
#include "../../Common/kvbuffer.h"
#include "../../Common/error.h"
#include "../../Common/queries.h"




const char* PLUGIN_NAME = "check_ntmem";



bool checkMemory(int& state, char* msg, const char* prefix, int size_or_percent, int free_mb, int total_mb, int wThreshold, int cThreshold)
{

  if (size_or_percent == PERCENT)
  {
    int p = (int)(free_mb / (double)total_mb * 100);
    int used_mb = total_mb - free_mb;

    char temp[128];
    snprintf(temp, 128, "%dMb (%d%s) of %dMb used (%dMb free)", used_mb, p, "%%", total_mb, free_mb);
    snprintf(msg, 128, "%s: Problem: %s", prefix, temp);

    debug("Msg: %s", msg);
    if (p < cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p < wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      snprintf(msg, 128, "%s: %s", prefix, temp);
    }
  }
  else if (size_or_percent == SIZE)
  {
    int p = free_mb;
    int used_mb = total_mb - free_mb;

    char temp[128];
    snprintf(temp, 128, "%dMb of %dMb used (%dMb free)", used_mb, total_mb, free_mb);
    snprintf(msg, 128, "%s: Problem: %s", prefix, temp);

    if (p < cThreshold)
    {
      state = STATE_CRITICAL;
    }
    else if (p < wThreshold)
    {
      state = STATE_WARNING;
    }
    else
    {
      snprintf(msg, 128, "%s: %s", prefix, temp);
    }
  }
  else
  {
    state = STATE_UNKNOWN;
    snprintf(msg, 128, "Unknown option !");
  }

  return true;
}




bool CheckCmdLine(int argc, char **argv, KVBuffer& buffer)
{
  if (argc != 9)
  {
    cout << "Incorrect arguments supplied\n\n";
    cout << "Usage: " << argv[0] << " <IPaddress> <port> <mMethod> <wmThresh> <cmThresh> <sMethod> <wsThresh> <csThresh>\n\n";
    cout << "Note:\n";
    cout << " <mMethod>     = Must be PERCENT or SIZE" << endl;
    cout << "                    say if the memory limits are in PERCENT or MB" << endl;
    cout << " <wmThresh>    = value of free memory that result in a WARNING message," << endl;
    cout << " <cmThresh>    = value of free memory that result in a CRITICAL message," << endl;
    cout << " <sMethod>     = Must be PERCENT or SIZE" << endl;
    cout << "                    say if the swap limits are in PERCENT or MB" << endl;
    cout << " <wsThresh>    = value of free swap that result in a WARNING message," << endl;
    cout << " <csThresh>    = value of free swap that result in a CRITICAL message," << endl << endl;
    cout << "Ex : "<<argv[0]<<" 10.2.0.1 1245 SIZE 23 12 PERCENT 80 90"<<endl<<endl;

    return false;
  }

  char id[10];
  sprintf(id, "%d", MEMUSAGE_QUERY);
  buffer.addKeyValue("id", id);

  return true;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuffer, KVBuffer& rbuffer, int& state, char* msg)
{
  state = STATE_OK;

  int mem_free_mb = rbuffer.getIntValue("free_phys");
  int mem_total_mb = rbuffer.getIntValue("total_phys");

  int wMemThreshold = atoi(argv[4]);
  int cMemThreshold = atoi(argv[5]);

  debug("Mem free: %d / Mem total: %d / wThresh: %d / cThresh: %d", mem_free_mb, mem_total_mb, wMemThreshold, cMemThreshold);

  // Memory
  int size_or_percent = ( strcmp(argv[3], "SIZE") == 0 ? SIZE : PERCENT );
  char memMsg[128];
  int memState = STATE_OK;
  if (!checkMemory(memState, memMsg, "Physical", size_or_percent, mem_free_mb, mem_total_mb, wMemThreshold, cMemThreshold))
  {
    memState = STATE_UNKNOWN;
  }

  if (memState == STATE_UNKNOWN)
  {
    state = memState;
    return false;
  }







  int swap_free_mb = rbuffer.getIntValue("free_swap");
  int swap_total_mb = rbuffer.getIntValue("total_swap");

  int wSwapThreshold = atoi(argv[7]);
  int cSwapThreshold = atoi(argv[8]);

  debug("Swap free: %d / Swap total: %d / wThresh: %d / cThresh: %d", swap_free_mb, swap_total_mb, wSwapThreshold, cSwapThreshold);

  // Swap
  char swapMsg[128];
  int swapState = STATE_OK;
  if (!checkMemory(swapState, swapMsg, "Swap", size_or_percent, swap_free_mb, swap_total_mb, wSwapThreshold, cSwapThreshold))
  {
    swapState = STATE_UNKNOWN;
    return false;
  }

  if (swapState == STATE_UNKNOWN)
  {
    state = swapState;
    return false;
  }





  // Synthesis

  // Critical
  state = STATE_OK;
  if (  memState == STATE_CRITICAL ||
        swapState == STATE_CRITICAL )
  {
    state = STATE_CRITICAL;
  }
  else if ( memState == STATE_WARNING ||
            swapState == STATE_WARNING )
  {
    state = STATE_WARNING;
  }

  sprintf(msg, "%s / %s", memMsg, swapMsg);

  return true;
}

