/*******************************************************************
*
* check_ntyourself plugin
* Copyright (c) 2000 Jan Christian Kaldestad
*
*
* Description:
* 	 Checks the processor load average on NT servers.
*	 Also finds the process with highest cpu usage.
*
******************************************************************/





#include "util.h"
#include "../../Common/kvbuffer.h"
#include "../../Common/error.h"
#include "../../Common/queries.h"



const char* PLUGIN_NAME = "check_ntself";

const char* version = VERSION;

bool CheckCmdLine(int argc, char **argv, KVBuffer& buffer)
{
  if (argc != 5)
  {
    cout << "Incorrect arguments supplied\n\n";
    cout << "Usage: " << argv[0] << " <IPaddress> <port> <wThresh> <cThresh>\n\n";
    cout << "Note:\n";
    cout << " <wThresh>      = value of free memory that result in a WARNING message," << endl;
    cout << " <cThresh>      = value of free memory that result in a CRITICAL message," << endl;

    return false;
  }

  char id[10];
  sprintf(id, "%d", SELFUSAGE_QUERY);
  buffer.addKeyValue("id", id);

  return true;
}


bool GetStringAndState(int argc, char** argv, KVBuffer& sbuffer, KVBuffer& rbuffer, int& state, char* output)
{
  char msg[1024];
  int used = rbuffer.getIntValue("used");
  int initial = rbuffer.getIntValue("initial");
  const char* nsserv_version = rbuffer.getValue("version");
  if (nsserv_version == 0)
  {
    nsserv_version = "<Unknown>";
  }

  int wThreshold = atoi(argv[3]);
  int cThreshold = atoi(argv[4]);

  int times = (int)(100 * (used / (double)initial));

  debug("Used: %d / Initial: %d / Times: %d%s / wThresh: %d / cThresh: %d", used, initial, times, "%%", wThreshold, cThreshold);

  state = STATE_OK;

  sprintf(msg, "Memory used: %d%s (now=%2.1fMb / init=%2.1fMb)", times, "%%", (double)((double)used/1024), (double)((double)initial/1024) );
  debug("Msg: %s", msg);
  if (times > cThreshold*10)
  {
    state = STATE_CRITICAL;
  }
  else if (times > wThreshold*10)
  {
    state = STATE_WARNING;
  }

  snprintf(output, 1024, "%s - NSServicer: v%s / Plugin: v%s", msg, nsserv_version, version);
  return true;
}
