/*******************************************************************
*
* check_ntdisk plugin
* Copyright (c) 2000 Jan Christian Kaldestad
*
*
* Description:
* 	 Checks the processor load average on NT servers.
*	 Also finds the process with highest cpu usage.
*
******************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <sys/errno.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>

#include "util.h"
#include "../../Common/error.h"
#include "../../Common/kvbuffer.h"
#include "../../Common/queries.h"





void SocketTimeoutHandler(int);
int AnalayzeMessage(char *ibuf, int &cpuLoad1, int &cpuLoad2, int &prosLoad, char *prosName);
int GetPasswd(char *passwd,int &passwdLen);
int CheckLoad(int cpuLoad2, int wload2, int cload2);
void PrintMsg(int state, int interval1, int cpuLoad1, int interval2, int cpuLoad2, char* prosName, int prosLoad);
bool getIPAddress(const char* host, struct in_addr& inaddr);



extern const char* PLUGIN_NAME;


int main(int argc, char **argv)
{
  int sockfd;
  char* ibuf = 0;
  char obuf[BUF_SIZE+1];
  char prosName[300];
  char passwd[MAX_PASSWD_LEN];
  int  passwdLen;
  struct sockaddr_in servaddr;
  int servPort;
  char host[128];

  KVBuffer sbuffer;
  KVBuffer rbuffer;

  int err;
  int size_or_percent;
  char path[256];
  int wThreshold;
  int cThreshold;


  debug("Starting (version=%s)", VERSION);

  if ( argc > 1 &&
        (strcmp(argv[1], "--version") == 0 ||
        strcmp(argv[1], "-v") == 0) )
  {
    cout << endl<<PLUGIN_NAME<<" plugin version V"<<VERSION<<endl<<endl;
    return STATE_UNKNOWN;
  }

  // Extern
  bool ok = CheckCmdLine(argc, argv, sbuffer);
  if(!ok)
  {
    return STATE_UNKNOWN;
  }
  debug("Cmd line OK");

  // Get password from passordfile
  if( GetPasswd(passwd, passwdLen) == STATE_CRITICAL )
  {
    cout << "Wrong password";
    return STATE_CRITICAL;
  }
  debug("Pwd OK (%s(size=%d))", passwd, passwdLen);
  sbuffer.addKeyValue("key", passwd);


  strcpy(host, argv[1]);
  servPort = atoi(argv[2]); // Port
  if(servPort < 1025 | servPort > 65535)
  {
    debug("Invalid port (%d). Must be a number between 1025 and 65535", servPort);
    return STATE_UNKNOWN;
  }


  memset(&servaddr, 0, sizeof(servaddr));
  servaddr.sin_family = AF_INET;
  servaddr.sin_port = htons(servPort);

  // Determine IP if arg is machine name
  if ( !getIPAddress(host, servaddr.sin_addr) )
  {
    cout << "Invalid host name or IP address ("<<host<<")\n";
    return STATE_CRITICAL;
  }
  debug("IP Address OK");




  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd == -1)
  {
    printf("Connextion refused\n");
    return STATE_CRITICAL;
  }
  err = connect(sockfd, (SA*) &servaddr, sizeof(servaddr));
  if(err<0)
  {
    printf("connect : %s\n", strerror(errno));
    return STATE_CRITICAL;
  }


  if ( !sbuffer.getString(obuf, sizeof(obuf)) )
  {
    debug("Can't make string to send");
    return STATE_UNKNOWN;
  }

  debug("Sending : '%s' (bytes=%d)", obuf, strlen(obuf) );
  if(!Send(sockfd, obuf))
  {
    cout << "ERROR: Send()" << endl;
    return STATE_CRITICAL;
  }


  int n = 0;
  if (!Receive(sockfd, ibuf, n))
  {
  	printf("Receive() error! : %s\n", strerror(errno) );
  	return STATE_CRITICAL;
  }
  debug("Received : '%s'(bytes=%d)", ibuf, n);
  shutdown(sockfd, 2);

  if( n < 0 )
  {
    printf("No data received! : %s\n", strerror(errno) );
    return STATE_UNKNOWN;
  }

  if (!rbuffer.initialize(ibuf))
  {
    return STATE_UNKNOWN;
  }


  int state = STATE_UNKNOWN;
  char msg[256];
  strcpy(msg, "UNKNOWN Status");
  if (!GetStringAndState(argc, argv, sbuffer, rbuffer, state, msg))
  {
    printf(msg);
    return STATE_UNKNOWN;
  }

  debug("End of prog : %d => %s", state, msg);
  printf(msg);
  return state;
}
