

#include "util.h"
#include "../../Common/error.h"
#include "../../Common/kvbuffer.h"



#include <netdb.h>




void SocketTimeoutHandler(int sig) 
{
	cout << "Socket timeout after " << SOCKET_TIMEOUT << " seconds" << endl;	
	exit(STATE_CRITICAL);
}

int GetPasswd(char *passwd, int &passwdLen) 
{
  int fd;

  char path[128];
  strcpy(path, CONFIG_DIR);
  strcat(path, PASSWD_FILE);
  fd = open(path, O_RDONLY);

  if( fd == -1 )
  {
    if( errno == ENOENT )
    {
      cout << "Error: Password file missing" << endl;
    }
    else
    {
      cout << "Error: " << strerror(errno) << endl;
    }

    return STATE_CRITICAL;
  }

  passwdLen = read(fd, passwd, MAX_PASSWD_LEN);
  if(passwdLen == -1)
  {
    cout << strerror(errno) << endl;
    close(fd);
    exit(1);
  }

  close(fd);

  passwd[passwdLen-1] = '\0';

  // Remove trailing "\n"
  passwdLen = strlen(passwd);
  while ( passwd[passwdLen-1] == '\n' ||
          passwd[passwdLen-1] == '\r' )
  {
    passwd[passwdLen-1] = 0;
    passwdLen--;
    debug("Removing trailing \\n in passwd");
  }

  return STATE_OK;
}

bool Send(int &sockfd, const char* obuf)
{
	int  n;
	char in;

	int nBytesToSend = strlen(obuf);

        int index = 0;
	while(nBytesToSend > 0)
	{
		n = send( sockfd, &obuf[index], strlen(obuf)-index, 0);
		nBytesToSend -= n;
                index += n;

		debug("Send: obuf: %s", obuf);
		if(nBytesToSend <= 0)
		{
			return true;
		}

		if(n < 0)
		{
		    	cout << strerror(errno) << endl;
			return false;
		}

		signal(SIGALRM, SocketTimeoutHandler);
		alarm(SOCKET_TIMEOUT);

		n = recv( sockfd, &in, BUF_SIZE, 0);
		alarm(0);

		if( n < 0 )
		{
	    		cout << strerror(errno) << endl;
		    	return false;
		}
	}

	return true;
}

bool Receive(int &sockfd, char*& ibuf, int& nBytesRecv)
{
	int 	index = 0;
	nBytesRecv = 0;
    int 	n;
    char  	ch='1';
    char 	*buffer;
    int		nBytes;

    char tmp[201];
    int tmp_size = 200;

    int nbRecv = 0;

	signal(SIGALRM, SocketTimeoutHandler);
	alarm(SOCKET_TIMEOUT);

	memset(tmp,0, tmp_size);
	n = recv( sockfd, tmp, tmp_size, 0);
	tmp[n]='\0';
        nbRecv++;
	nBytesRecv = n;

	debug("\nReceive: ibuf(nBytesRecv=%d): %s", nBytesRecv, tmp);

	// reset the alarm
	alarm(0);

	if( n < 0 )
	{
		cout << strerror(errno) << endl;
		return false;
	}

        nBytes = nBytesRecv;
	if( strncmp(tmp, "nbytes=", 7) == 0)
	{
		nBytes = atoi(&tmp[7]);
  	}

	ibuf = new char[nBytes+1];
	memset(ibuf, 0, nBytes);
	strncpy(ibuf, tmp, nBytesRecv);

	if( strncmp(tmp, "nbytes=", 7) == 0)
	{
                debug("We need %d bytes to get all the message", nBytes);
		if( nBytes < tmp_size)
		{
			return true;
		}
	}
	else
	{
        	debug("No info on nBytes (UDP?)");
        	return true;
	}

	while(1)
	{
		// we are ready to receive next message
		//send(sockfd, ibuf, 1, 0);

		signal(SIGALRM, SocketTimeoutHandler);
		alarm(SOCKET_TIMEOUT);

		// wait for next message
		debug("Waiting for next buffer...");
		memset(tmp, 0, tmp_size);
		n = recv( sockfd, tmp, tmp_size, 0);
		debug("I received %d bytes", n);
		tmp[n] = 0;
		nBytesRecv += n;
		strncat(ibuf, tmp, n);

		debug("Received: ibuf(nBytesRecv=%d nBytes=%d): %s", nBytesRecv, nBytes, tmp);

		alarm(0);
		if ( n < 0)
		{
			cout << "Receive error: " << strerror(errno) << endl;
			return false;
		}

                if (n == 0)
		{
	                if (nBytesRecv != nBytes)
                        {
                        	debug("recv returned 0 (all bytes there) but message is not complete");
                        }
                        else
                        {
	                	debug("OK, I received the entire message");
                  	}
			return true;
		}
	}
	return false;
}



void DEBUG(const char* format, ...)
{
  // Construct a string
  static char theString[2048];
  theString[0] = 0;
  va_list arglist;
  va_start(arglist, format);
  vsnprintf(theString, 2048, format, arglist);

  fprintf(stderr, theString);
  fprintf(stderr, "\n");
  fflush(stderr);
}
void NO_DEBUG(const char* format, ...)
{
}


bool getIPAddress(const char* host, struct in_addr& inaddr)
{
  debug("getIPAddress: Host: %s", host);

  const char* toFind = host;
  bool allNumbers = true;
  while(toFind[0] != 0)
  {
    if (  toFind[0] < 48 ||
          toFind[0] > 58 )
    {
      allNumbers = false;
      break;
    }
    toFind++;
  }
  debug("All numbers : %d", allNumbers);

  if (allNumbers)
  {
    // It's an address, return it
    debug("getIPAddress : already an address");
  	bool err = inet_aton(host, &inaddr);
    return err;
  }

  // It's a host, get by addr
  debug("Calling gethostbyname(%s)", host);
  struct hostent* s_host = gethostbyname(host);
  if (s_host == 0)
  {
    const char* msg = hstrerror(h_errno);
    debug("gethostname : %s", msg);
    return false;
  }

  memcpy(&inaddr, s_host->h_addr_list[0], s_host->h_length );

  debug("getIPAddress : OK");
  return true;
}
